//############################################################################
//
// EZFB ~ Linux Frame Buffer API ~
//
// by James Lehman
// james@akrobiz.com
//
// began: Feb. 2001
//
// EZFB is intended to give C programmers an easy-to-use library of functions
// for drawing points, lines and strings of characters into the Linux frame
// buffer. It also provides a means of displaying bitmap images on any portion
// of the screen and capturing the contents of any portion of the screen to
// a standard bitmap graphics file. All of this is embodied in a set of
// functions that are polymorphic with regard to the characteristics of the
// frame buffer and the bitmap files used. In other words, it makes no
// difference what the screen height or width is or what the color depth is,
// all of the function calls are the same and will have appropriate and
// predictable effects.
//
// This software is distributed in the hope that it will be useful, but
// without any warranty; without even the implied warranty of merchantability
// or fitness for a particular purpose. See the GNU General Public License
// for more details.
//
//############################################################################
#ifdef __cplusplus
extern "C"
{
#endif        /* __cplusplus */

#ifndef __EZFB_PIXEL_DRAWING_FUNCTIONS__
#define __EZFB_PIXEL_DRAWING_FUNCTIONS__

//############################################################################
#include "ezfb_types.h"
#include "ezfb_colors.h"

//############################################################################
int      ezfb_put_line          (struct ezfb* fb, int x1, int y1, int  x2, int  y2, int c);
int      ezfb_put_ray           (struct ezfb* fb, int  x, int  y, float r, float t, int c);
int      ezfb_put_rectangle     (struct ezfb* fb, int x1, int y1, int  x2, int  y2, int c, int filled);

//############################################################################





//############################################################################
int ezfb_put_pixel__1bit(struct ezfb* fb, u_int  x, u_int  y, u_int  c);
int ezfb_put_pixel__2bit(struct ezfb* fb, u_int  x, u_int  y, u_int  c);
int ezfb_put_pixel__4bit(struct ezfb* fb, u_int  x, u_int  y, u_int  c);
int ezfb_put_pixel__8bit(struct ezfb* fb, u_int  x, u_int  y, u_int  c);
int ezfb_put_pixel_16bit(struct ezfb* fb, u_int  x, u_int  y, u_int  c);
int ezfb_put_pixel_24bit(struct ezfb* fb, u_int  x, u_int  y, u_int  c);
int ezfb_put_pixel_32bit(struct ezfb* fb, u_int  x, u_int  y, u_int  c);

//############################################################################
u_int ezfb_get_pixel__1bit(struct ezfb* fb, u_int  x, u_int  y);
u_int ezfb_get_pixel__2bit(struct ezfb* fb, u_int  x, u_int  y);
u_int ezfb_get_pixel__4bit(struct ezfb* fb, u_int  x, u_int  y);
u_int ezfb_get_pixel__8bit(struct ezfb* fb, u_int  x, u_int  y);
u_int ezfb_get_pixel_16bit(struct ezfb* fb, u_int  x, u_int  y);
u_int ezfb_get_pixel_24bit(struct ezfb* fb, u_int  x, u_int  y);
u_int ezfb_get_pixel_32bit(struct ezfb* fb, u_int  x, u_int  y);

//############################################################################
extern inline u_short ezfb_get_r_from_pixel(struct ezfb* fb, u_int  x, u_int  y)
{
    if(fb->exists && (x < fb->Var.xres_virtual) && (y < fb->Var.yres_virtual))
    {
        if(fb->Var.bits_per_pixel <= 8)
            return fb->colors.red[fb->get_pixel(fb, x, y)];
        return ezfb_get_r_from_rgb(fb, fb->get_pixel(fb, x, y));
    }
    return 0;
}

//############################################################################
extern inline u_short ezfb_get_g_from_pixel(struct ezfb* fb, u_int  x, u_int  y)
{
    if(fb->exists && (x < fb->Var.xres_virtual) && (y < fb->Var.yres_virtual))
    {
        if(fb->Var.bits_per_pixel <= 8)
            return fb->colors.green[fb->get_pixel(fb, x, y)];
        return ezfb_get_g_from_rgb(fb, fb->get_pixel(fb, x, y));
    }
    return 0;
}

//############################################################################
extern inline u_short ezfb_get_b_from_pixel(struct ezfb* fb, u_int  x, u_int  y)
{
    if(fb->exists && (x < fb->Var.xres_virtual) && (y < fb->Var.yres_virtual))
    {
        if(fb->Var.bits_per_pixel <= 8)
            return fb->colors.blue[fb->get_pixel(fb, x, y)];
        return ezfb_get_b_from_rgb(fb, fb->get_pixel(fb, x, y));
    }
    return 0;
}

//############################################################################
#endif

#ifdef __cplusplus
}
#endif        /* __cplusplus */
//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################



