//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#ifndef __EZ_ILDA_DEFINITIONS__
#define __EZ_ILDA_DEFINITIONS__

//###################################################################################
#include "ez_ilda_frame_set.hpp"
#include "ez_palette_set.hpp"

//############################################################################
typedef void(*ez_ilda_frame_effect    )(ez_ilda_frame*    );
typedef void(*ez_ilda_frame_set_effect)(ez_ilda_frame_set*);

//###################################################################################
class ez_ilda : public ez_ilda_frame_set, public ez_palette_set
{
public:
    ez_ilda    ();
   ~ez_ilda    ();
    //-------------------------------------------------------------------------------
    ez_ilda_frame& operator [] (int index) // does not set frame_index
                            {
                                if(number_of_frames())
                                {
                                    if(index >= number_of_frames())
                                        index = 0;
                                    if(index < 0)
                                        index = number_of_frames() - 1;
                                    return ez_ilda_frame_set::at(index);
                                }
                                return null_frame;
                            };
    //-------------------------------------------------------------------------------
    ez_ilda& operator += (const ez_ilda_frame_set& frames)
                            {
                                ez_ilda_frame_set::insert(ez_ilda_frame_set::end(), frames.begin(), frames.end());
                                renumber();
                                return *this;
                            };
    //-------------------------------------------------------------------------------
    ez_ilda& operator  = (const ez_ilda_frame_set& frames)
                            {
                                ez_ilda_frame_set::clear();
                                ez_ilda_frame_set::insert(ez_ilda_frame_set::end(), frames.begin(), frames.end());
                                renumber();
                                return *this;
                            };
    //-------------------------------------------------------------------------------
    ez_ilda& operator += (const ez_ilda_frame& frame)
                            {
                                ez_ilda_frame_set::push_back(frame);
                                renumber();
                                return *this;
                            };
    //-------------------------------------------------------------------------------
    ez_ilda& operator += (const ez_palette_set& palettes)
                            {
                                ez_palette_set::insert(ez_palette_set::end(), palettes.begin(), palettes.end());
                                return *this;
                            };
    //-------------------------------------------------------------------------------
    ez_ilda& operator += (const ez_palette& palette)
                            {
                                ez_palette_set::push_back(palette);
                                return *this;
                            };    
    //-------------------------------------------------------------------------------
    void           tell() const
                        {
                            ez_ilda_frame_set::tell();
                            ez_palette_set::tell();
                            return;
                        };
    //-------------------------------------------------------------------------------
    void           stats() const
                        {
                            ez_ilda_frame_set::tell();
                            ez_palette_set::tell();
                            return;
                        };
    //-------------------------------------------------------------------------------
    void           clear_frames()
                                {
                                    ez_ilda_frame_set::clear();
                                    return;
                                };
    //-------------------------------------------------------------------------------
    void           clear_palettes()
                                {
                                    ez_palette_set::clear();
                                    return;
                                };


    //-------------------------------------------------------------------------------
    void           simplify_viewing_angle()
								{
									while(viewing_angle.x < 0)
										viewing_angle.x += two_pi;
									while(viewing_angle.x > two_pi)
										viewing_angle.x -= two_pi;

									while(viewing_angle.y < 0)
										viewing_angle.y += two_pi;
									while(viewing_angle.y > two_pi)
										viewing_angle.y -= two_pi;

									while(viewing_angle.z < 0)
										viewing_angle.z += two_pi;
									while(viewing_angle.z > two_pi)
										viewing_angle.z -= two_pi;
								};
    //-------------------------------------------------------------------------------
    void           sync_rgb_and_palette   ();
    //-------------------------------------------------------------------------------
    void           display                ();
    //-------------------------------------------------------------------------------
    void           load_background_bitmap (ezfb_bitmap* bmp);
    void           load_background_bitmap (string file     );
    void           clear_background_bitmap();
    //-------------------------------------------------------------------------------
    void           load_frame_effects     ();
    void           load_frame_set_effects ();
    void           list_frame_effects     ();
    void           list_frame_set_effects ();
    void           call_frame_effect      (string name);
    void           call_frame_set_effect  (string name);
    void           load_effects           ()
                                            {
                                                load_frame_effects();
                                                load_frame_set_effects();
                                            };
    //-------------------------------------------------------------------------------
    ezfb           fb         ; // the frame buffer (video card)
    ezfb_bitmap    bmp        ; // a memory clone of fb
    ezfb_bitmap    background ; // a memory clone of fb

    int            wave_sample_rate       ,
                   lit_dwell_overhang     ,
                   font_size_factor       ,
                   output_bmp_size        ;

    ez_3D_float    arbitrary_point        ,
                   viewing_angle          ,
                   viewing_scale          ,
                   viewing_offset         ;

    double         distance_delta_max     ,
                   longest_dwell_microsec ,
                   insignificant_distance ,
                   zero_angle_tolerance   ,
                   rendered_arc_angle     , // in degrees
                   frames_per_second      ,
                   still_frame_duration   ;

    bool           key_validated          ,
                   show_points            ,
                   show_blanking          ,
                   show_cursors           ,
                   show_palettes          ,
                   show_stats             ,
                   show_fixed_bounds      ,
                   show_fixed_origin      ,
                   show_floating_axis     ,
                   show_floating_bounds   ,
				   show_arbitrary_point   ,
                   show_effects_generation,
                   auto_scale_dxf         ,
                   maintain_dxf_origin    ,
                   black_dwell_points     ,
                   auto_flatten_z         ,
                   auto_optimize          ;
    //-------------------------------------------------------------------------------
    vector<string>                        frame_effect_names     ,
                                          frame_set_effect_names ;
    //-------------------------------------------------------------------------------
    vector<ez_ilda_frame_effect>          frame_effects          ;
    vector<ez_ilda_frame_set_effect>      frame_set_effects      ;
    //-------------------------------------------------------------------------------
};

//###################################################################################
extern ez_ilda* ez_ilda_pointer; // the one and only!

//###################################################################################
#endif

//###################################################################################
/////////////////////////////////////////////////////////////////////////////////////
//###################################################################################



