//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA file format driver in C++      ~
// ~             built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#include "ez_ilda_common.hpp"
    
//############################################################################
const double pi                = (4 * atan(1.0))  ;
const double two_pi            = (pi * 2.0)       ;
const double quarter_pi        = (pi / 4.0)       ;
const double half_pi           = (pi / 2.0)       ;
const double three_quarters_pi = (3 * quarter_pi) ;
const double one_degree        = (pi / 180.0)     ;
const double one_radian        = (180.0 / pi)     ;

//############################################################################
bool get_dxf_pair(ifstream& in, int& group_code, string& entity_string)
{
    char line[1025];
    in >> group_code;
    if(group_code != -1)
    {
        while(isspace(in.peek()))
            in.get(); // eat the '\r''\n'

        in.getline(line, 1024); // dxf limit is 256
        entity_string = line;

        int i = entity_string.size() - 1;
        while(!isgraph(entity_string[i]))
            i--;

        entity_string = entity_string.substr(0, i + 1);
    }
#if 0 // EZ_ILDA_DEBUG_MESSAGES
    cout << "group_code     {" << group_code    << "}" << ((group_code) ? ("") : (" <<--- group start ---<<")) << endl
         << "entity_string  {" << entity_string << "}"<< endl;
#endif
    return (group_code != 0 && !in.eof());
}

//############################################################################
void ez_ilda_terminate(int sig)
{
    ezfb_terminate(sig);
}

//############################################################################
string call_command(string command, void(*my_exit)(int), bool tee)
{
    FILE   *pipe;
    string  command_return;
    char c;
    //------------------------------------------------------------------------
    if(tee)
        command += " 2>&1"; // redirect stderr to stdout.
    pipe = popen(command.c_str(), "r");
    if(pipe == NULL)
    {
        cerr << "system call to {"
             << command
             << "} failed."
             << endl;
        //cerr.flush();
        my_exit(10); // must exit process
        exit(-1);    // just in case
    }
    //------------------------------------------------------------------------
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "call    {" << command << "}" << endl;
    cout << "returns {";
#endif
    while(1)
    {
        c = fgetc(pipe);
        if(c == EOF)
            break;
#if EZ_ILDA_DEBUG_MESSAGES
        cout << c;
#endif
        command_return += c;
    }
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "}" << endl;
#endif
    pclose(pipe);
    return command_return;
}

//############################################################################
void show_tag()
{
    cout << "\n\n"
    "#####################################\n"
    "#####################################\n"
    "##                                 ##\n"
    "##  LaserBoy                       ##\n"
    "##                                 ##\n"
    "##  10-??-03   to   03-21-08       ##\n"
    "##                                 ##\n"
    "##  by James Lehman                ##\n"
    "##     james@akrobiz.com           ##\n"
    "##     Extra Stimulus Inc.         ##\n"
    "##     www.akrobiz.com             ##\n"
    "##                                 ##\n"
    "#####################################\n"
    "#####################################\n" << endl;
    return;
}

//############################################################################
struct in_addr ez_IP_address(const char *if_name)
{
    struct ifreq ifr;
    struct sockaddr_in *addr;
    int s;
    s = socket(AF_INET, SOCK_DGRAM, 0);
    if(s == -1)
    {
        perror("socket");
        exit(1);
    }
    strcpy(ifr.ifr_name, if_name);
    if(ioctl(s, SIOCGIFADDR, &ifr) < 0)
    {
        perror("ioctl(SIOCGIFADDR)");
        exit(1);
    }
    addr = (struct sockaddr_in *) &ifr.ifr_addr;
    close(s);
    return addr->sin_addr;
}


//############################################################################
string ez_MAC_address_string()
{
    string command_call("ifconfig eth0"),
           command_return(call_command(command_call, ez_ilda_terminate, false));
    command_return = command_return.substr((command_return.find_first_of('\n') - 19), 17);
    return command_return;
}

/*
//############################################################################
static int ez_MAC_address(unsigned int ipaddr, struct sockaddr *hwaddr)
{
    struct ifreq *ifr, *ifend;
    unsigned int ina, mask;
    struct ifreq ifreq;
    struct ifconf ifc;
    struct ifreq ifs[MAX_IFS];
    int s;

    s = socket(AF_INET, SOCK_DGRAM, 0);

    ifc.ifc_len = sizeof(ifs);
    ifc.ifc_req = ifs;
    if(ioctl(s, SIOCGIFCONF, &ifc) < 0)
    {
        printf("ioctl(SIOCGIFCONF): %m\n");
        return 0;
    }
    // Scan through looking for an interface with an Internet
    // address on the same subnet as `ipaddr'.
    ifend = ifs + (ifc.ifc_len / sizeof(struct ifreq));
    for(ifr = ifc.ifc_req; ifr < ifend; ifr++)
    {
        if(ifr->ifr_addr.sa_family == AF_INET)
        {
            ina = ((struct sockaddr_in *)&ifr->ifr_addr)->sin_addr.s_addr;
            strncpy(ifreq.ifr_name, ifr->ifr_name, sizeof(ifreq.ifr_name));
            // Check that the interface is up, and not point-to-point nor loopback.
            if(ioctl(s, SIOCGIFFLAGS, &ifreq) < 0)
                continue;
            if(((ifreq.ifr_flags ^ FLAGS_GOOD) & FLAGS_MASK) != 0)
                continue;
            // Get its netmask and check that it's on the right subnet.
            if(ioctl(s, SIOCGIFNETMASK, &ifreq) < 0)
                continue;
            mask = ((struct sockaddr_in *)&ifreq.ifr_addr)->sin_addr.s_addr;
            if(((ipaddr ^ ina) & mask) != 0)
                continue;
            break;
        }
    }
    if(ifr >= ifend)
        return 0;

    // Finally get the hardware address.
    memset (&ifreq.ifr_hwaddr, 0, sizeof(struct sockaddr));
    if(ioctl(s, SIOCGIFHWADDR, &ifreq) < 0)
    {
        printf("SIOCGIFHWADDR(%s): %m\n", ifreq.ifr_name);
        return 0;
    }
    memcpy (hwaddr, &ifreq.ifr_hwaddr, sizeof (struct sockaddr));

    printf(   "MacGate: Using Device %s, IP %s -> Ethernet %02x:%02x:%02x:%02x:%02x:%02x\n"
            , ifreq.ifr_name, in_ntoa(ina)
            , (int) ((unsigned char *) &ifreq.ifr_hwaddr.sa_data)[0]
            , (int) ((unsigned char *) &ifreq.ifr_hwaddr.sa_data)[1]
            , (int) ((unsigned char *) &ifreq.ifr_hwaddr.sa_data)[2]
            , (int) ((unsigned char *) &ifreq.ifr_hwaddr.sa_data)[3]
            , (int) ((unsigned char *) &ifreq.ifr_hwaddr.sa_data)[4]
            , (int) ((unsigned char *) &ifreq.ifr_hwaddr.sa_data)[5]
          );
    return 1;
}
*/
//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################



