//-----------------------------------------------------//
//    C++ Sound Card Interface Class                   //
//    by James Lehman                                  //
//    Jan. 2002                                        //
//-----------------------------------------------------//

#include "ez_oss_dsp.hpp"

//#################################################################################
signed short get_max_short(signed short* shorts, int count)
{
    signed short max = 0;
    for(int i = 0; i < count; i++)
    {
        if(shorts[i] > max)
            max = shorts[i];
    }
    return max;
}

//#################################################################################







//#################################################################################
oss_dsp::oss_dsp(int num_channels, int sample_rate, int bits_per_sample)
{    
    open_dsp();
    //set_full_duplex();
    set_16bps        ();
    set_num_channels (num_channels);
    set_rate         (sample_rate);
}

//#################################################################################
oss_dsp::~oss_dsp()
{    
    close_dsp();
}

//#################################################################################
bool oss_dsp::open_dsp()
{    
    if((dsp_fd = open(DSP_NAME, O_RDWR, 0)) == -1)
    {
        perror(DSP_NAME);
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::close_dsp()
{    
    close(dsp_fd);
    return true;
}

//#################################################################################
bool oss_dsp::set_full_duplex()
{
    int bit_mask;
    ioctl(dsp_fd, SNDCTL_DSP_GETCAPS, &bit_mask);
    if(!(bit_mask & DSP_CAP_DUPLEX))
    {
        printf(DSP_NAME " does not support full duplex\n");
        return false;
    }
    else if(ioctl(dsp_fd, SNDCTL_DSP_SETDUPLEX, 0) == -1)
    {
        perror("SNDCTL_DSP_SETDUPLEX");
        return false;
    }
    else
        return true;
}

//#################################################################################
bool oss_dsp::set_16bps()
{
    int format = AFMT_S16_LE;
    if(ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &format) == -1)
        perror("SNDCTL_DSP_SETFMT");
    if(format != AFMT_S16_LE)
    {
        printf(DSP_NAME " does not support signed 16\n");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_8bps()
{
    int format = AFMT_S8;
    if(ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &format) == -1)
        perror("SNDCTL_DSP_SETFMT");
    if(format != AFMT_S8)
    {
        printf(DSP_NAME " does not support signed 8\n");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_num_channels(int channels)
{
    int want = channels;
    if(ioctl(dsp_fd, SNDCTL_DSP_CHANNELS, &want) == -1)
        perror("SNDCTL_DSP_CHANNELS");
    if(channels != want)
    {
        printf(DSP_NAME " does not support %d channels\n", channels);
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_rate(int rate)
{
    int sample_rate = rate;
    if(ioctl(dsp_fd, SNDCTL_DSP_SPEED, &sample_rate) == -1)
        perror("SNDCTL_DSP_SPEED");
    if(sample_rate != rate)
    {
        printf(DSP_NAME " does not support %dHz sample rate", rate);
        return false;
    }
    return true;
}   

//#################################################################################








//#################################################################################
int oss_dsp::get_odelay()
{
    int odelay;
    if(ioctl(dsp_fd, SNDCTL_DSP_GETODELAY, &odelay) == -1)
        perror("SNDCTL_DSP_GETODELAY");
    return odelay;
}

//#################################################################################
int oss_dsp::get_driver_frag_size()
{
    int frag_size;
    if(ioctl(dsp_fd, SNDCTL_DSP_GETBLKSIZE, &frag_size) == -1)
        perror("SNDCTL_DSP_GETBLKSIZE");

    return frag_size;
}

//#################################################################################







//#################################################################################
bool oss_dsp::set_vol(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_VOLUME), &arg) == -1)
    {
        perror("SOUND_MIXER_VOLUME");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_pcm(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_PCM), &arg) == -1)
    {
        perror("SOUND_MIXER_PCM");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_line(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_LINE), &arg) == -1)
    {
        perror("SOUND_MIXER_LINE");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_bass(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_BASS), &arg) == -1)
    {
        perror("SOUND_MIXER_BASS");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_treble(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_TREBLE), &arg) == -1)
    {
        perror("SOUND_MIXER_TREBLE");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_synth(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_SYNTH), &arg) == -1)
    {
        perror("SOUND_MIXER_SYNTH");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_pc_speaker(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_SPEAKER), &arg) == -1)
    {
        perror("SOUND_MIXER_SPEAKER");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_mic(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_MIC), &arg) == -1)
    {
        perror("SOUND_MIXER_MIC");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_CD(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_CD), &arg) == -1)
    {
        perror("SOUND_MIXER_CD");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_imix(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_IMIX), &arg) == -1)
    {
        perror("SOUND_MIXER_IMIX");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_altpcm(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_ALTPCM), &arg) == -1)
    {
        perror("SOUND_MIXER_ALTPCM");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_rec_lev(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_RECLEV), &arg) == -1)
    {
        perror("SOUND_MIXER_RECLEV");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_in_gain(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_IGAIN), &arg) == -1)
    {
        perror("SOUND_MIXER_IGAIN");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_out_gain(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_OGAIN), &arg) == -1)
    {
        perror("SOUND_MIXER_OGAIN");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_line_1(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_LINE1), &arg) == -1)
    {
        perror("SOUND_MIXER_LINE1");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_line_2(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_LINE2), &arg) == -1)
    {
        perror("SOUND_MIXER_LINE2");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_line_3(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_LINE3), &arg) == -1)
    {
        perror("SOUND_MIXER_LINE3");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_digital_1(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_DIGITAL1), &arg) == -1)
    {
        perror("SOUND_MIXER_DIGITAL1");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_digital_2(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_DIGITAL2), &arg) == -1)
    {
        perror("SOUND_MIXER_DIGITAL2");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_digital_3(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_DIGITAL3), &arg) == -1)
    {
        perror("SOUND_MIXER_DIGITAL3");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_phone_in(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_PHONEIN), &arg) == -1)
    {
        perror("SOUND_MIXER_PHONEIN");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_phone_out(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_PHONEOUT), &arg) == -1)
    {
        perror("SOUND_MIXER_PHONEOUT");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_video(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_VIDEO), &arg) == -1)
    {
        perror("SOUND_MIXER_VIDEO");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_radio(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_RADIO), &arg) == -1)
    {
        perror("SOUND_MIXER_RADIO");
        return false;
    }
    return true;
}

//#################################################################################
bool oss_dsp::set_monitor(int volume)
{
    int arg = pack_vol(volume, volume);
    if(ioctl(dsp_fd, MIXER_WRITE(SOUND_MIXER_MONITOR), &arg) == -1)
    {
        perror("SOUND_MIXER_MONITOR");
        return false;
    }
    return true;
}

//#################################################################################









//#################################################################################
int oss_dsp::get_vol()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_VOLUME), &volume) == -1)
        perror("SOUND_MIXER_VOLUME");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_bass()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_BASS), &volume) == -1)
        perror("SOUND_MIXER_BASS");
    return volume;
}

//#################################################################################
int oss_dsp::get_treble()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_TREBLE), &volume) == -1)
        perror("SOUND_MIXER_TREBLE");
    return volume;
}

//#################################################################################
int oss_dsp::get_synth()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_SYNTH), &volume) == -1)
        perror("SOUND_MIXER_SYNTH");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_pcm()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_PCM), &volume) == -1)
        perror("SOUND_MIXER_PCM");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_speaker()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_SPEAKER), &volume) == -1)
        perror("SOUND_MIXER_SPEAKER");
    return volume;
}

//#################################################################################
int oss_dsp::get_line()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_LINE), &volume) == -1)
        perror("SOUND_MIXER_LINE");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_mic()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_MIC), &volume) == -1)
        perror("SOUND_MIXER_MIC");
    return volume;
}

//#################################################################################
int oss_dsp::get_cd()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_CD), &volume) == -1)
        perror("SOUND_MIXER_CD");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_imix()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_IMIX), &volume) == -1)
        perror("SOUND_MIXER_IMIX");
    return volume;
}

//#################################################################################
int oss_dsp::get_altpcm()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_ALTPCM), &volume) == -1)
        perror("SOUND_MIXER_ALTPCM");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_reclev()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_RECLEV), &volume) == -1)
        perror("SOUND_MIXER_RECLEV");
    return volume;
}

//#################################################################################
int oss_dsp::get_igain()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_IGAIN), &volume) == -1)
        perror("SOUND_MIXER_IGAIN");
    return volume;
}

//#################################################################################
int oss_dsp::get_ogain()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_OGAIN), &volume) == -1)
        perror("SOUND_MIXER_OGAIN");
    return volume;
}

//#################################################################################
int oss_dsp::get_line1()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_LINE1), &volume) == -1)
        perror("SOUND_MIXER_LINE1");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_line2()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_LINE2), &volume) == -1)
        perror("SOUND_MIXER_LINE2");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_line3()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_LINE3), &volume) == -1)
        perror("SOUND_MIXER_LINE3");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_digital1()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_DIGITAL1), &volume) == -1)
        perror("SOUND_MIXER_DIGITAL1");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_digital2()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_DIGITAL2), &volume) == -1)
        perror("SOUND_MIXER_DIGITAL2");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_digital3()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_DIGITAL3), &volume) == -1)
        perror("SOUND_MIXER_DIGITAL3");
    return unpack_l(volume);
}

//#################################################################################
int oss_dsp::get_phonein()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_PHONEIN), &volume) == -1)
        perror("SOUND_MIXER_PHONEIN");
    return volume;
}

//#################################################################################
int oss_dsp::get_phoneout()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_PHONEOUT), &volume) == -1)
        perror("SOUND_MIXER_PHONEOUT");
    return volume;
}

//#################################################################################
int oss_dsp::get_video()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_VIDEO), &volume) == -1)
        perror("SOUND_MIXER_VIDEO");
    return volume;
}

//#################################################################################
int oss_dsp::get_radio()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_RADIO), &volume) == -1)
        perror("SOUND_MIXER_RADIO");
    return volume;
}

//#################################################################################
int oss_dsp::get_monitor()
{
    int volume;
    if(ioctl(dsp_fd, MIXER_READ(SOUND_MIXER_MONITOR), &volume) == -1)
        perror("SOUND_MIXER_MONITOR");
    return volume;
}

//#################################################################################










//#################################################################################
void oss_dsp::show_mixer()
{
    int bitmask;
    if(ioctl(dsp_fd, SOUND_MIXER_READ_DEVMASK, &bitmask) == -1)
        perror("SOUND_MIXER_READ_DEVMASK");
    else
    {
        printf("\n");
        if(bitmask & SOUND_MASK_VOLUME   ) printf("     vol      : %d\n", get_vol      ());
        if(bitmask & SOUND_MASK_BASS     ) printf("     bass     : %d\n", get_bass     ());
        if(bitmask & SOUND_MASK_TREBLE   ) printf("     treble   : %d\n", get_treble   ());
        if(bitmask & SOUND_MASK_SYNTH    ) printf("     synth    : %d\n", get_synth    ());
        if(bitmask & SOUND_MASK_PCM      ) printf("     pcm      : %d\n", get_pcm      ());
        if(bitmask & SOUND_MASK_SPEAKER  ) printf("     speaker  : %d\n", get_speaker  ());
        if(bitmask & SOUND_MASK_LINE     ) printf("     line     : %d\n", get_line     ());
        if(bitmask & SOUND_MASK_MIC      ) printf("     mic      : %d\n", get_mic      ());
        if(bitmask & SOUND_MASK_CD       ) printf("     cd       : %d\n", get_cd       ());
        if(bitmask & SOUND_MASK_IMIX     ) printf("     imix     : %d\n", get_imix     ());
        if(bitmask & SOUND_MASK_ALTPCM   ) printf("     altpcm   : %d\n", get_altpcm   ());
        if(bitmask & SOUND_MASK_RECLEV   ) printf("     reclev   : %d\n", get_reclev   ());
        if(bitmask & SOUND_MASK_IGAIN    ) printf("     igain    : %d\n", get_igain    ());
        if(bitmask & SOUND_MASK_OGAIN    ) printf("     ogain    : %d\n", get_ogain    ());
        if(bitmask & SOUND_MASK_LINE1    ) printf("     line1    : %d\n", get_line1    ());
        if(bitmask & SOUND_MASK_LINE2    ) printf("     line2    : %d\n", get_line2    ());
        if(bitmask & SOUND_MASK_LINE3    ) printf("     line3    : %d\n", get_line3    ());
        if(bitmask & SOUND_MASK_DIGITAL1 ) printf("     digital1 : %d\n", get_digital1 ());
        if(bitmask & SOUND_MASK_DIGITAL2 ) printf("     digital2 : %d\n", get_digital2 ());
        if(bitmask & SOUND_MASK_DIGITAL3 ) printf("     digital3 : %d\n", get_digital3 ());
        if(bitmask & SOUND_MASK_PHONEIN  ) printf("     phonein  : %d\n", get_phonein  ());
        if(bitmask & SOUND_MASK_PHONEOUT ) printf("     phoneout : %d\n", get_phoneout ());
        if(bitmask & SOUND_MASK_VIDEO    ) printf("     video    : %d\n", get_video    ());
        if(bitmask & SOUND_MASK_RADIO    ) printf("     radio    : %d\n", get_radio    ());
        if(bitmask & SOUND_MASK_MONITOR  ) printf("     monitor  : %d\n", get_monitor  ());
    }
    return;
}

//#################################################################################
void oss_dsp::show_caps()
{
    int bitmask;
    if(ioctl(dsp_fd, SNDCTL_DSP_GETCAPS, &bitmask) == -1)
        perror("SNDCTL_DSP_GETCAPS");
    else
    {
        printf("\n");
        if(bitmask & DSP_CAP_DUPLEX  ) printf("     full duplex\n");
        if(bitmask & DSP_CAP_REALTIME) printf("     realtime\n");
        if(bitmask & DSP_CAP_BATCH   ) printf("     batch storage\n");
        if(bitmask & DSP_CAP_COPROC  ) printf("     coprocessor\n");
        if(bitmask & DSP_CAP_TRIGGER ) printf("     trigger\n");
        if(bitmask & DSP_CAP_MMAP    ) printf("     mmap\n");
    }
    return;
}

//#################################################################################
///////////////////////////////////////////////////////////////////////////////////
//#################################################################################
























