//-----------------------------------------------------//
//    C++ Sound Card Interface Class                   //
//    by James Lehman                                  //
//    Jan. 2002                                        //
//-----------------------------------------------------//

#ifndef ___CPP_SOUND_CARD_CLASS___
#define ___CPP_SOUND_CARD_CLASS___

//#################################################################################
#include "ez_ilda_includes.hpp"
#include <sys/soundcard.h>

//#################################################################################
#define DSP_NAME                   "/dev/dsp"
#define DSP_PLAY_BUFFER_SIZE       4800 // signed short

//#################################################################################
class oss_dsp
{
public:
    oss_dsp( int num_channels    =     6,
             int sample_rate     = 96000,
             int bits_per_sample =    16
           );
    //-----------------------------------------------------------------------------
   ~oss_dsp();
    //-----------------------------------------------------------------------------
    bool open_dsp        ();
    bool close_dsp       ();
    bool set_full_duplex ();

    bool set_16bps       ();
    bool set_8bps        ();
    bool set_num_channels(int channels);
    bool set_rate        (int rate);

    int  get_odelay      ();
    int  get_driver_frag_size();
    //-----------------------------------------------------------------------------
    int pack_vol(int l, int r)  {
                                    if (l <   0)    l =   0;
                                    if (r <   0)    r =   0;
                                    if (l > 100)    l = 100;
                                    if (r > 100)    r = 100;
                                    return l | (r << 8);
                                };
    //-----------------------------------------------------------------------------
    int pack_lr(int l, int r)   {    return l | (r << 8);           };
    int unpack_l(int vol)       {    return (vol & 0xff);           };
    int unpack_r(int vol)       {    return ((vol >> 8) & 0xff);    };
    //-----------------------------------------------------------------------------
    bool set_vol         (int volume);
    bool set_pcm         (int volume);
    bool set_line        (int volume);
    bool set_bass        (int volume);
    bool set_treble      (int volume);
    bool set_synth       (int volume);
    bool set_pc_speaker  (int volume);
    bool set_mic         (int volume);
    bool set_CD          (int volume);
    bool set_imix        (int volume);
    bool set_altpcm      (int volume);
    bool set_rec_lev     (int volume);
    bool set_in_gain     (int volume);
    bool set_out_gain    (int volume);
    bool set_line_1      (int volume);
    bool set_line_2      (int volume);
    bool set_line_3      (int volume);
    bool set_digital_1   (int volume);
    bool set_digital_2   (int volume);
    bool set_digital_3   (int volume);
    bool set_phone_in    (int volume);
    bool set_phone_out   (int volume);
    bool set_video       (int volume);
    bool set_radio       (int volume);
    bool set_monitor     (int volume);
    //-----------------------------------------------------------------------------
    int  get_vol         ();
    int  get_bass        ();
    int  get_treble      ();
    int  get_synth       ();
    int  get_pcm         ();
    int  get_speaker     ();
    int  get_line        ();
    int  get_mic         ();
    int  get_cd          ();
    int  get_imix        ();
    int  get_altpcm      ();
    int  get_reclev      ();
    int  get_igain       ();
    int  get_ogain       ();
    int  get_line1       ();
    int  get_line2       ();
    int  get_line3       ();
    int  get_digital1    ();
    int  get_digital2    ();
    int  get_digital3    ();
    int  get_phonein     ();
    int  get_phoneout    ();
    int  get_video       ();
    int  get_radio       ();
    int  get_monitor     ();
    //-----------------------------------------------------------------------------
    void show_mixer      ();
    void show_caps       ();
    //-----------------------------------------------------------------------------
    signed short get_max_short(signed short* shorts, int count);
    //-----------------------------------------------------------------------------
    signed short play_buffer[DSP_PLAY_BUFFER_SIZE];
    
    int           dsp_fd,
                  play_buffer_index,
                  bytes_in_buffer,
                  samples_in_buffer,
                  number_of_channels;
};

//#################################################################################

#endif

//#################################################################################
///////////////////////////////////////////////////////////////////////////////////
//#################################################################################



