//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA file format driver in C++      ~
// ~             built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#include "ez_ilda.hpp"
    
//############################################################################
int ez_palette::from_ifstream(ifstream& in, const ez_ilda_header& header)
{
    int         i,
                error = EZ_ILDA_OK;
    ez_color  color;
    clear();
    //------------------------------------------------------------------------
    for(i = 0; i < header.quantity; i++)
    {
        if((error = color.from_ifstream(in)))
            break;
        push_back(color);
    }
    find_factors();
    name = header.name;
    header.tell();
    return error;
}

//############################################################################
void ez_palette::to_ofstream(ofstream& out, ez_ilda_header& header) const
{
    int i;
    header.quantity = (u_short)number_of_colors();
    header.to_ofstream(out);
    for(i = 0; i < number_of_colors(); i++)
        at(i).to_ofstream(out);
    header.tell();
    return;
}

//############################################################################
int ez_palette::from_ascii_file(string file)
{
    int          color_count = 0,
                 color_value,
                 error = EZ_ILDA_OK;
    ez_color     color;
    string       junk;

    if(    file.size() < 4
        || file.substr(file.size() - 4) != ".pal"
      )
        file += ".pal";

    ifstream     in((EZ_ILDA_PALETTES_SHARE + file).c_str(), ios::in);

#if EZ_ILDA_OUTPUT_MESSAGES
    cout << "opening " << EZ_ILDA_PALETTES_SHARE + file << " ..." << endl;
#endif

    if(in.is_open())
    {
        in >> name;
        in >> junk; // Red
        in >> junk; // Green
        in >> junk; // Blue
        while(!in.eof() && (color_count < 256))
        {
            in >> color_value;    color.r = (char)color_value;
            in >> color_value;    color.g = (char)color_value;
            in >> color_value;    color.b = (char)color_value;
            push_back(color);
            color_count++;
        }
#if EZ_ILDA_OUTPUT_MESSAGES
        if((color_count == 256) && (!in.eof()))
            cout << "more than 256 colors, ignored" << endl;
#endif
        in.close();
        find_factors();
#if EZ_ILDA_OUTPUT_MESSAGES
        cout << EZ_ILDA_PALETTES_SHARE + file << " opened" << endl;
#endif
    }
#if EZ_ILDA_OUTPUT_MESSAGES
    else
    {
        cout << EZ_ILDA_PALETTES_SHARE + file << " failed to open for input" << endl;
        error = EZ_ILDA_FILE_OPEN_FAILED;
    }
#endif
    return error;
}

//############################################################################
void ez_palette::save_as_ascii(string file) const
{
    if(    file.size() < 4
        || file.substr(file.size() - 4) != ".pal"
      )
        file += ".pal";

#if EZ_ILDA_OUTPUT_MESSAGES
    cout << "saving " << EZ_ILDA_PALETTES_SHARE + file << " ..." << endl;
#endif
    ofstream out((EZ_ILDA_PALETTES_SHARE + file).c_str(), ios::out);
    if(out.is_open())
    {
        out << name
            << endl
            << "Red     Green   Blue"
            << endl;

        for(int i = 0; i < number_of_colors(); i++)
            out << setw(3)
                << (int)at(i).r
                << setw(4)
                << (int)at(i).g
                << setw(4)
                << (int)at(i).b
                << endl;

        out << endl;

        out.close();
#if EZ_ILDA_OUTPUT_MESSAGES
        cout << EZ_ILDA_PALETTES_SHARE + file << " saved" << endl;
#endif
    }
#if EZ_ILDA_OUTPUT_MESSAGES
    else
        cout << EZ_ILDA_PALETTES_SHARE + file << " failed to open for output" << endl;
#endif
    return;
}

//############################################################################
int ez_palette::from_ezfb_bmp_palette()
{
    int i;
    if(BMP.bpp == 8)
    {
        clear();
        resize(EZ_PALETTE_MAX);
        for(i = 0; i < EZ_PALETTE_MAX; i++)
            set_palette_item( i,
                              bmp_get_palette_index_r(&BMP, i),
                              bmp_get_palette_index_g(&BMP, i),
                              bmp_get_palette_index_b(&BMP, i)
                            );
        first = 0;
        last  = 255;
        black = BMP.black ;
        gray  = BMP.gray  ;
        white = BMP.white ;
        return EZ_ILDA_OK;
    }
    return EZ_ILDA_INCOMPATIBLE_FORMATS;
}

//############################################################################
int ez_palette::to_ezfb_bmp_palette() const
{
    return to_ezfb_bmp_palette(&BMP);
}

//############################################################################
int ez_palette::to_ezfb_bmp_palette(ezfb_bitmap* bmp) const
{
    int i;
    if(bmp->bpp == 8)
    {
        for(i = 0; i < EZ_PALETTE_MAX; i++)
            if(i < number_of_colors())
                bmp_set_palette_index(bmp, i, at(i).r, at(i).g, at(i).b);
            else
                bmp_set_palette_index(bmp, i, 0, 0, 0);
        bmp->black = black ;
        bmp->gray  = gray  ;
        bmp->white = white ;
        return EZ_ILDA_OK;
    }
    return EZ_ILDA_INCOMPATIBLE_FORMATS;
}

//############################################################################
int ez_palette::from_ezfb_palette()
{
    int i;
    if(FB.Var.bits_per_pixel == 8)
    {
        clear();
        resize(EZ_PALETTE_MAX);
        for(i = 0; i < EZ_PALETTE_MAX; i++)
        set_palette_item( i,
                          ezfb_get_r_from_cmap_index(&FB, i),
                          ezfb_get_g_from_cmap_index(&FB, i),
                          ezfb_get_b_from_cmap_index(&FB, i)
                        );
        first = 0;
        last  = 255;
        black = FB.colors.black ;
        gray  = FB.colors.gray  ;
        white = FB.colors.white ;
        return EZ_ILDA_OK;
    }
    return EZ_ILDA_INCOMPATIBLE_FORMATS;
}

//############################################################################
int ez_palette::to_ezfb_palette() const
{
    int i;
    if(FB.Var.bits_per_pixel == 8)
    {
        for(i = 0; i < EZ_PALETTE_MAX; i++)
            if(i < number_of_colors())
                ezfb_set_cmap_index(&FB, i, at(i).r << 16, at(i).g << 16, at(i).b << 16);
            else
                ezfb_set_cmap_index(&FB, i, 0, 0, 0);           
        FB.colors.black = black ;
        FB.colors.gray  = gray  ;
        FB.colors.white = white ;
        return EZ_ILDA_OK;
    }
    return EZ_ILDA_INCOMPATIBLE_FORMATS;
}

//############################################################################
void ez_palette::find_factors()
{
    int    i       ,
           j       ,
           k = 255 ;

    first = black = gray = white = last = 0;
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
    {
        if(at(black) > at(i))
          black = i;
    //------------------------------------------------------------------------
        if(at(white) < at(i))
          white = i;
    //------------------------------------------------------------------------
        j =   abs(at(i).r - 0x80)
            + abs(at(i).g - 0x80)
            + abs(at(i).b - 0x80);
        if(j < k)
        {
            gray = i;
            k    = j;
        }
    }
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
        if(at(i) > 0)
        {
            first = i;
            break;
        }
    //------------------------------------------------------------------------
    for(i = number_of_colors() - 1; i >= 0 ; i--)
        if(at(i) > 0)
        {
            last = i;
            break;
        }
    //------------------------------------------------------------------------
    return;
}

//############################################################################
int ez_palette::best_match(ez_color rgb) const
{
    int    i           ,
           j           ,
           k     = 255 ,
           match = 0   ;
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
    {
        j =   abs(at(i).r - rgb.r)
            + abs(at(i).g - rgb.g)
            + abs(at(i).b - rgb.b);
        if(j < k)
        {
            match = i;
            k     = j;
        } 
    }
    //------------------------------------------------------------------------
    return match;
}

//############################################################################
bool ez_palette::unite(const ez_palette& palette)
{
    if(    palette.number_of_colors() >  0
        && palette.number_of_colors() <= EZ_PALETTE_MAX
      )
    {
        int            i,
                       j;
        ez_palette     super_palette(*this);
        //-----------------------------------------------------------------
        for(i = 0; i < palette.number_of_colors(); i++)
        {
            for(j = 0; j < super_palette.number_of_colors(); j++)
                if(super_palette[j] == palette.at(i))
                    break;
            if(j == super_palette.number_of_colors())
                super_palette.push_back(palette.at(i));
        }
        //-----------------------------------------------------------------
        if(super_palette.number_of_colors() <= EZ_PALETTE_MAX)
        {
            super_palette.reorder();
            super_palette.find_factors();
#if EZ_ILDA_DEBUG_MESSAGES
            cout << palette.name << " united with " << super_palette.name << endl;
#endif
            *this = super_palette;
            return true;
        }
        //-----------------------------------------------------------------
        else
            return false;
    }
    //---------------------------------------------------------------------
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "ez_palette::unite called with palette of no size!" << endl;
#endif
    return false;
}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################




















