//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#ifndef __EZ_REAL_segment_DEFINITIONS__
#define __EZ_REAL_segment_DEFINITIONS__

//###################################################################################
#include "ez_palette.hpp"
#include "ez_ilda_point.hpp"
#include "ez_ilda_utility.hpp"

//###################################################################################
class ez_real_segment : public vector<ez_real_point>
{
public:
    ez_real_segment()
                 : palette_index (EZ_ILDA_DEFAULT),
                   error         (EZ_ILDA_OK     )
                                                        {}
    //-------------------------------------------------------------------------------
    ez_real_segment(const int& palette_index)
                 : palette_index (palette_index),
                   error         (EZ_ILDA_OK   )
                                                        {}
    //-------------------------------------------------------------------------------
    ez_real_segment(const int& palette_index, int not_used)
                 : palette_index (palette_index),
                   error         (EZ_ILDA_OK   )
					{
						push_back(ez_real_point());
						push_back(ez_real_point());
					}
    //-------------------------------------------------------------------------------
    ez_real_segment(const ez_real_segment& segment)
                 : palette_index (segment.palette_index),
                   error         (EZ_ILDA_OK         )
                    {
                        clear();
                        insert(end(), segment.begin(), segment.end());
                    }
    //-------------------------------------------------------------------------------
    ez_real_segment(ez_real_point from, ez_real_point to) // 3D line function
                 : palette_index      (EZ_ILDA_DEFAULT),
                   error              (EZ_ILDA_OK     )
                    {   // 3D line constructor
                        int steps = distance_points(to, from);
                        if(steps)
                        {
                            ez_3D_float  _to        (to),
                                         _from      (from),
                                         difference (_to - _from),
                                         delta      (difference / steps);
                            //-------------------------------------------------------
                            for(int i = 1; i < steps; i++)
                                push_back(ez_real_point( ez_3D_float(_from + (delta * i)),
                                                         (ez_color)to,
                                                         to.k,
                                                         to.c
                                                       )
                                         );
                            //-------------------------------------------------------
                        }
                    }
    //-------------------------------------------------------------------------------
   ~ez_real_segment()    {    clear();    }
    //-------------------------------------------------------------------------------
    inline bool is_2D() const
                {
                    if(number_of_points() > 1)
                    {
                        for(int i = 0; i < number_of_points(); i++)
                            if(at(i).z != 0.0)
                                return false;
                    }
                    return true;
                }
    //-------------------------------------------------------------------------------
    inline ez_real_segment&  operator += (const ez_real_segment& segment)
                                {
                                    insert(end(), segment.begin(), segment.end());
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_segment&  operator += (const ez_real_point& point)
                                {
                                    push_back(point);
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_segment&  operator += (const ez_3D_float& float_3D)
                                {
                                    for(int i = 0; i < (int)size(); i++)
										at(i) += float_3D;
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_segment&  operator *= (const ez_3D_float& float_3D)
                                {
                                    for(int i = 0; i < (int)size(); i++)
										at(i) *= float_3D;
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_segment   operator + (const ez_3D_float& float_3D)
                                {
									ez_real_segment real_segment;
                                    for(int i = 0; i < (int)size(); i++)
										real_segment.push_back(at(i) + float_3D);
                                    return real_segment;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_segment   operator * (const ez_3D_float& float_3D)
                                {
									ez_real_segment real_segment;
                                    for(int i = 0; i < (int)size(); i++)
										real_segment.push_back(at(i) * float_3D);
                                    return real_segment;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_segment&  operator =  (const ez_real_segment& segment)
                                {
                                    clear();
                                    insert(begin(), segment.begin(), segment.end());
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point  first_lit_point() const
                    {
                        for(int i = 1; i < number_of_points(); i++)
                            if(at(i).is_lit())
                                return at(i);
                        return ez_real_point(0);
                    }
    //-------------------------------------------------------------------------------
    inline int number_of_points     () const {   return (int)size();   }
    //-------------------------------------------------------------------------------
    inline int first_lit_point_index() const
            {
                for(int i = 1; i < number_of_points(); i++)
                    if(at(i).is_lit())
                        return i;
                return -1;
            }
    //-------------------------------------------------------------------------------
    inline ez_real_point first_lit_anchor() const
                    {
                        for(int i = 1; i < number_of_points(); i++)
                            if(at(i).is_lit())
                                return at(i - 1);
                        return ez_real_point(0);
                    }
    //-------------------------------------------------------------------------------
    inline int first_lit_anchor_index() const
            {
                for(int i = 1; i < number_of_points(); i++)
                    if(at(i).is_lit())
                        return (i - 1);
                return -1;
            }
    //-------------------------------------------------------------------------------
    inline ez_real_point last_lit_anchor() const
                    {
                        for(int i = number_of_points() - 1; i > 0; i--)
                            if(at(i).is_lit())
                                return at(i - 1);
                        return ez_real_point(0);
                    }
    //-------------------------------------------------------------------------------
    inline ez_real_point last_lit_point() const
                    {
                        for(int i = number_of_points() - 1; i > 0; i--)
                            if(at(i).is_lit())
                                return at(i);
                        return ez_real_point(0);
                    }
    //-------------------------------------------------------------------------------
    inline int last_lit_point_index() const
            {
                for(int i = number_of_points() - 1; i > 0; i--)
                    if(at(i).is_lit())
                        return i;
                return -1;
            }
    //-------------------------------------------------------------------------------
    inline int number_of_lit_points() const
            {
                int count = 0;
                for(int i = 1; i < number_of_points(); i++)
                    if(at(i).is_lit())
                        count++;
                return count;
            }
    //-------------------------------------------------------------------------------
    inline int number_of_blank_points() const
            {
                int count = 0;
                for(int i = 1; i < number_of_points(); i++)
                    if(at(i).is_blank())
                        count++;
                return count;
            }
    //-------------------------------------------------------------------------------
    inline int number_of_unique_colors() const
        {
            if(number_of_points() > 1)
            {
                int           i, j;
                ez_palette  palette;
                palette.push_back((ez_color)first_lit_point());
                for(i = first_lit_point_index() + 1; i < number_of_points(); i++)
                {
                    for(j = 0; j < palette.number_of_colors(); j++)
                    {
                        if(palette[j] == (ez_color)at(i))
                            break;
                    }
                    if(j == palette.number_of_colors())
                        palette.push_back(at(i));
                }
                return palette.number_of_colors();
            }
        }
    //-------------------------------------------------------------------------------
    inline ez_palette as_ez_palette() const
                {
                    ez_palette palette(number_of_points());
                    for(int i = 0; i < number_of_points(); i++)
                        palette[i] = (ez_color)(at(i));
                    return palette;
                }
    //-------------------------------------------------------------------------------
    void              tell                         (                               ) const ;
    ez_3D_float       segment_front                (                               ) const ;
    ez_3D_float       segment_back                 (                               ) const ;
    ez_3D_float       segment_top                  (                               ) const ;
    ez_3D_float       segment_bottom               (                               ) const ;
    ez_3D_float       segment_right                (                               ) const ;
    ez_3D_float       segment_left                 (                               ) const ;
    float             segment_height               (                               ) const ;
    float             segment_width                (                               ) const ;
    float             segment_depth                (                               ) const ;
    float             segment_size                 (                               ) const ;
    ez_real_segment&  reverse                      (                               )       ;
    void              unblank_all                  (                               )       ;
    void              flip                         (int plane                      )       ;
    void              quarter_turn                 (int plane, int turns           )       ;
    void              z_order_points               (short span                     )       ;
    void              flatten_z                    (                               )       ;
    int               rotate                       (ez_3D_float a                  )       ;
    int               rotate_around_origin         (ez_3D_float a                  )       ;
	int               rotate_around_point          (ez_3D_float p, ez_3D_float a   )       ;
    void              rotate_colors                (                               )       ;
    bool              find_rgb_in_palette          (const ez_palette& palette      )       ;
    void              set_rgb_from_palette         (                               )       ;
    void              set_palette_to_332           (                               )       ;
    void              sync_rgb_and_palette         (                               )       ;
    void              bit_reduce_to_palette        (                               )       ;
    int               move                         (ez_3D_float d                  )       ;
    int               scale                        (ez_3D_float s                  )       ;
    int               scale_around_point           (ez_3D_float p, ez_3D_float s   )       ;
    int               scale_around_origin          (ez_3D_float s                  )       ;
    ez_3D_float       find_center                  (                               ) const ;
    int               number_of_segments           (                               ) const ;
    bool              find_segment                 (int index, int& start, int& end) const ;
    ez_real_segment   find_segment                 (int index                      ) const ;
    ez_3D_float       find_segment_center          (int index                      ) const ;
    int               move_segment                 (int index, ez_3D_float f       )       ;
    int               rotate_segment               (int index, ez_3D_float a       )       ;
    int               rotate_segment_around_origin (int index, ez_3D_float a       )       ;
    int               scale_segment                (int index, ez_3D_float m       )       ;
    int               scale_segment_around_origin  (int index, ez_3D_float m       )       ;
    //-------------------------------------------------------------------------------
    int               palette_index ,
                      error         ;
    //-------------------------------------------------------------------------------
};

//###################################################################################
#endif

//###################################################################################
/////////////////////////////////////////////////////////////////////////////////////
//###################################################################################





