//################################################################################
//
// laserboy.cpp  (formerly ild2fb.cpp)
//
// ~   International Laser Display Association    ~
// ~ ILDA file display and transformations in C++ ~
// ~               built for EZFB                 ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began : October 2003
// name changed to laserboy : January 2005
//
//################################################################################
/*
    EZFB_ARROW_UP              0x415b1b
    EZFB_ARROW_DOWN            0x425b1b
    EZFB_ARROW_RIGHT           0x435b1b
    EZFB_ARROW_LEFT            0x445b1b
    EZFB_ESCAPE_KEY            0x1b
EZFB_F1_KEY                0x504f1b
EZFB_F2_KEY                0x514f1b
EZFB_F3_KEY                0x524f1b
EZFB_F4_KEY                0x534f1b
    EZFB_DELETE_KEY            0x7f
    EZFB_CTRL_A                0x1
    EZFB_CTRL_B                0x2
	EZFB_CTRL_D                0x4
    EZFB_CTRL_E                0x5
    EZFB_CTRL_F                0x6
	EZFB_CTRL_G                0x7
    EZFB_BACKSPACE_KEY         0x8  EZFB_CTRL_H
    EZFB_TAB_KEY               0x9  EZFB_CTRL_I
    EZFB_ENTER_KEY             0xa  EZFB_CTRL_J
	EZFB_CTRL_K                0xb
EZFB_CTRL_L                0xc
EZFB_CTRL_M                0xd
EZFB_CTRL_N                0xe
	EZFB_CTRL_O                0xf
    EZFB_CTRL_P                0x10
EZFB_CTRL_R                0x12
EZFB_CTRL_T                0x14
    EZFB_CTRL_U                0x15
	EZFB_CTRL_V                0x16
EZFB_CTRL_W                0x17
EZFB_CTRL_X                0x18
EZFB_CTRL_Y                0x19
*/

#include "ez_ilda.hpp"

//################################################################################
#define LASERBOY_DEMO 0
#define CHARS_IN_LINE 80

//################################################################################
ez_ilda_point A(int point, int frame_index) // single frame, 40 points, 3D spring
{
    ez_ilda_point P;
    float factor = (point / 8.0) * two_pi;
    P.x = (short int)(900 * sin(factor));
    P.y = (short int)(900 * cos(factor));
    P.z = (short int)(point * 300 - 6000);
    P.k = 0;
    P.c = 1;
    return P;
}

//################################################################################
ez_ilda_point B(int point, int frame_index)
{
    ez_ilda_point P;
    float factor = (point / 20.0) * two_pi,
          shift  = (two_pi / 60.0) * frame_index;
    P.x = (short int)(point * 1200);
    P.y = (short int)(sin(3.5 * factor + shift) * point * 200);
    P.z = (short int)(0);
    P.k = 0;
    P.c = point * 2 + 2;
    return P;
}

//################################################################################
// The functions A, B and F are definitions for all points in an ez_ilda_frame
// and possibly all frames in an animated frame set.
// There is are ctors for ez_ilda_frame and for ez_ilda_frame_set
// ez_ilda_frame(ez_ilda_point(*F)(int, int), int points_per_frame, int frame_index);
// ez_ilda_frame_set(ez_ilda_point(*F)(int, int), int points_per_frame, int num_frames);
// When calling these the total number of points and frames is defined.
//--------------------------------------------------------------------------------
ez_ilda_point F(int point, int frame_index)
{
    ez_ilda_point P;
    float factor = (point / 699.0) * 30 * pi;
    P.x = (short int)(22000 * sin(factor) * (sin(31 * factor / 15)) );
    P.y = (short int)(22000 * cos(factor) * (sin(31 * factor / 15)) );
    P.z = (short int)((point * 40) - 14000);
    P.k = 0;
    P.c = point % 253 + 2;
    return P;
}


//################################################################################
void show_version()
{
	cout << "03.21.2008   build 1";
	return;
}

//################################################################################
void show_hot_keys()
{
 // "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
    cout << EZ_ANSI_BOLD EZ_ANSI_CYAN
	                "\n\n\n\n\n\n"
	EZ_ANSI_RED     "[ARROW_LEFT]|[ARROW_RIGHT] : +|- frame\n"
    EZ_ANSI_YELLOW  "  [ARROW_UP]|[ARROW_DOWN]  : +|- target palette \n"
	EZ_ANSI_GREEN   "     CTRL_F | CTRL_G       : open | save file data\n"
	EZ_ANSI_CYAN   	"     CTRL_B | CTRL_E       : go to begining | end frame\n"
	EZ_ANSI_BLUE    " `|~   |   -|_   |  =|+    : animate | rotate | both (forward | reverse)\n"
	EZ_ANSI_MAGENTA "  (         |        )     : toggle group frame select   start | end\n"
	EZ_ANSI_RED     "         [SPACE]           : toggle individual frame select\n"
    EZ_ANSI_YELLOW  "        [TAB_KEY]          : global application settings\n"
	EZ_ANSI_GREEN   "       [BACKSPACE]         : delete current frame (no selection neccessary)\n"
	EZ_ANSI_CYAN   	"          CTRL_O           : observe display artifacts\n"
	EZ_ANSI_BLUE    "          CTRL_V           : apply view\n"
	EZ_ANSI_MAGENTA "          CTRL_A           : alter frame(s) contents order and orientation\n"
	EZ_ANSI_RED     "          CTRL_D           : draw, move, and recolor points\n"
    EZ_ANSI_YELLOW  "          CTRL_K           : move, scale and rotate segments\n"
	EZ_ANSI_GREEN   "          CTRL_P           : select and apply palette transformations\n"
	EZ_ANSI_CYAN   	"          CTRL_U           : universaly alter contents and order of frame set\n"
	EZ_ANSI_BLUE    "            z              : show viewing parameters and list view hotkeys\n"
	EZ_ANSI_MAGENTA "            h              : display this list of hotkeys\n"
    EZ_ANSI_RED     "          [ESC]            : escape an action | exit program"
#if !LASERBOY_DEMO
                                                                                 " (saving OUT.ild)"
#endif
	EZ_ANSI_RESET
	<< endl;
    return;
}

//################################################################################
void list_ilda_files()
{
    string command_call("ls -1 *.ild");
    chdir(EZ_ILDA_FRAMES_SHARE.c_str());
    cout << endl << "list of ilda files:" << endl;
    cout << call_command(command_call, ez_ilda_terminate, false) << endl << endl;
    chdir("../");
    return;    
}

//################################################################################
void list_wave_files()
{
    string command_call("ls -1 *.wav");
    chdir(EZ_ILDA_WAVES_SHARE.c_str());
    cout << endl << "list of wave files:" << endl;
    cout << call_command(command_call, ez_ilda_terminate, false) << endl << endl;
    chdir("../");
    return;    
}

//################################################################################
void list_palette_files()
{
    string command_call("ls -1 *.pal");
    chdir(EZ_ILDA_PALETTES_SHARE.c_str());
    cout << endl << "list of palette files:" << endl;
    cout << call_command(command_call, ez_ilda_terminate, false) << endl << endl;
    chdir("../");
    return;    
}

//################################################################################
void list_dxf_files()
{
    string command_call("ls -1 *.dxf");
    chdir(EZ_ILDA_DXF_SHARE.c_str());
    cout << endl << "list of dxf files:" << endl;
    cout << call_command(command_call, ez_ilda_terminate, false) << endl << endl;
    chdir("../");
    return;    
}

//################################################################################
void list_dxf_directories()
{
    string command_call("ls -1p --ignore '*.dxf'");
    chdir(EZ_ILDA_DXF_SHARE.c_str());
    cout << endl << "list of directories containing dxf files:" << endl;
    cout << call_command(command_call, ez_ilda_terminate, false) << endl << endl;
    chdir("../");
    return;    
}

//################################################################################
void list_bmp_files()
{
    string command_call("ls -1 *.bmp");
    chdir(EZ_ILDA_BMP_SHARE.c_str());
    cout << endl << "list of bmp files:" << endl;
    cout << call_command(command_call, ez_ilda_terminate, false) << endl << endl;
    chdir("../");
    return;    
}

//################################################################################
void list_bmp_directories()
{
    string command_call("ls -1p --ignore '*.bmp'");
    chdir(EZ_ILDA_BMP_SHARE.c_str());
    cout << endl << "list of directories containing bmp files:" << endl;
    cout << call_command(command_call, ez_ilda_terminate, false) << endl << endl;
    chdir("../");
    return;    
}

//################################################################################
void list_cpp_files()
{
    string command_call("ls -1 *.cpp");
    chdir(EZ_ILDA_CPP_SHARE.c_str());
    cout << call_command(command_call, ez_ilda_terminate, false) << endl;
    chdir("../");
    return;    
}

//################################################################################
void wipe_input_string(char* input_string)
{
	int i;
	for(i = 0; i <= CHARS_IN_LINE; i++)
		input_string[i] = '\0';
	return;
}

//################################################################################
int main(int argc, char** argv)
{
    show_tag();
    cout << "[" << ez_MAC_address_string() << "]" << endl;

    ez_ilda      ild; // this is it!

    ez_3D_float  temp_3D_float;

    u_int        key;

    char         input_string[CHARS_IN_LINE + 1];

    bool         application_is_running = true,
                 loop_animate           = true,
                 approximate_frame_rate = true;

    int          move_points_per_key    = 100;

    float        temp_float,
                 recolor_span_factor    = 2.0;

    double                   ticks_per_second = cpu_speed();
    unsigned long long int   time_zero;

    if(argc > 1)
        ild.from_file(argv[1]);
    else
        ild.from_file("in.ild");

    if(ild.error)
        cout << "\napplication started with NO DATA!" << endl;    

    ild.display();
    show_hot_keys();
	show_version();

    cout << " .... Go!" << endl;

	while(application_is_running)
	{
		while((key = ezfb_get_key()) != EZFB_ESCAPE_KEY) // from ezfb_tty.c
		{   // ezfb_get_key() accepts a single key stroke with no return
			switch(key)
			{   //----------------------------------------------------------------
				default:    cout << "hit the 'h' key for help!" << endl;
							break;
				//----------------------------------------------------------------
				case 'h':   show_hot_keys();
							break;
				//----------------------------------------------------------------








				//----------------------------------------------------------------
				case '1':   ild.viewing_angle.x += one_degree;
							break;
				//----------------------------------------------------------------
				case '!':   ild.viewing_angle.x -= one_degree;
							break;
				//----------------------------------------------------------------
				case '2':   ild.viewing_angle.y += one_degree;
							break;
				//----------------------------------------------------------------
				case '@':   ild.viewing_angle.y -= one_degree;
							break;
				//----------------------------------------------------------------
				case '3':   ild.viewing_angle.z += one_degree;
							break;
				//----------------------------------------------------------------
				case '#':   ild.viewing_angle.z -= one_degree;
							break;
				//----------------------------------------------------------------
				case '4':   ild.viewing_angle += one_degree;
							break;
				//----------------------------------------------------------------
				case '$':   ild.viewing_angle -= one_degree;
							break;

				//----------------------------------------------------------------
				case '5':   ild.viewing_angle   = 0;
							cout << "front" << endl;
							break;
				//----------------------------------------------------------------
				case '%':   ild.viewing_angle.x = 0;
							ild.viewing_angle.y = pi;
							ild.viewing_angle.z = 0;
							cout << "back" << endl;
							break;
				//----------------------------------------------------------------
				case '6':   ild.viewing_angle.x = 0;
							ild.viewing_angle.y = half_pi;
							ild.viewing_angle.z = 0;
							cout << "left side" << endl;
							break;
				//----------------------------------------------------------------
				case '^':   ild.viewing_angle.x = 0;
							ild.viewing_angle.y = -half_pi;
							ild.viewing_angle.z = 0;
							cout << "right side" << endl;
							break;
				//----------------------------------------------------------------
				case '7':   ild.viewing_angle.x = half_pi;
							ild.viewing_angle.y = 0;
							ild.viewing_angle.z = 0;
							cout << "top" << endl;
							break;
				//----------------------------------------------------------------
				case '&':   ild.viewing_angle.x = -half_pi;
							ild.viewing_angle.y = 0;
							ild.viewing_angle.z = 0;
							cout << "bottom" << endl;
							break;
				//----------------------------------------------------------------
				case '8':   ild.viewing_angle += one_degree;
							if(    loop_animate
								|| (ild.frame_index < ild.number_of_frames() - 1)
							  )
								++ild;
							break;
				//----------------------------------------------------------------
				case '*':   ild.viewing_angle -= one_degree;
							if(    loop_animate
								|| (ild.frame_index > 0)
							  )
								--ild;
							break;
				//----------------------------------------------------------------
				case 'q':   ild.viewing_offset.x += 200;
							break;
				//----------------------------------------------------------------
				case 'Q':   ild.viewing_offset.x -= 200;
							break;
				//----------------------------------------------------------------
				case 'w':   ild.viewing_offset.y += 200;
							break;
				//----------------------------------------------------------------
				case 'W':   ild.viewing_offset.y -= 200;
							break;
				//----------------------------------------------------------------
				case 'e':   ild.viewing_offset.z += 200;
							break;
				//----------------------------------------------------------------
				case 'E':   ild.viewing_offset.z -= 200;
							break;
				//----------------------------------------------------------------
				case 'r':   ild.viewing_offset += 200;
							break;
				//----------------------------------------------------------------
				case 'R':   ild.viewing_offset -= 200;
							break;
				//----------------------------------------------------------------
				case 't':
				case 'T':   ild.viewing_offset = 0.0;
							cout << "on center" << endl;
							break;
				//----------------------------------------------------------------
				case 'a':   ild.viewing_scale.x *= 1.01;
							break;
				//----------------------------------------------------------------
				case 'A':   ild.viewing_scale.x *= 0.99;
							break;
				//----------------------------------------------------------------
				case 's':   ild.viewing_scale.y *= 1.01;
							break;
				//----------------------------------------------------------------
				case 'S':   ild.viewing_scale.y *= 0.99;
							break;
				//----------------------------------------------------------------
				case 'd':   ild.viewing_scale.z *= 1.01;
							break;
				//----------------------------------------------------------------
				case 'D':   ild.viewing_scale.z *= 0.99;
							break;
				//----------------------------------------------------------------
				case 'f':   ild.viewing_scale *= 1.01;
							break;
				//----------------------------------------------------------------
				case 'F':   ild.viewing_scale *= 0.99;
							break;
				//----------------------------------------------------------------
				case 'g':
				case 'G':   ild.viewing_scale = 1.0;
							cout << "actual size" << endl;
							break;
				//----------------------------------------------------------------
				case 'z':   cout << EZ_ANSI_BOLD
									"\n\n\n\n\n\n"
					 EZ_ANSI_YELLOW "    1|! : +|- rotate X      " EZ_ANSI_CYAN  "5|% : front | back\n"
					 EZ_ANSI_YELLOW "    2|@ : +|- rotate Y      " EZ_ANSI_CYAN  "6|^ :  left | right\n"
					 EZ_ANSI_YELLOW "    3|# : +|- rotate Z      " EZ_ANSI_CYAN  "7|& :   top | bottom\n"
					 EZ_ANSI_YELLOW "    4|$ : +|- rotate XYZ    " EZ_ANSI_CYAN  "8|* : +|- rotate animate\n\n"

					EZ_ANSI_MAGENTA "    q|Q : +|- pan  X        " EZ_ANSI_GREEN "a|A : +|- zoom X\n"
					EZ_ANSI_MAGENTA "    w|W : +|- pan  Y        " EZ_ANSI_GREEN "s|S : +|- zoom Y\n"
					EZ_ANSI_MAGENTA "    e|E : +|- pan  Z        " EZ_ANSI_GREEN "d|D : +|- zoom Z\n"
					EZ_ANSI_MAGENTA "    r|R : +|- pan  XYZ      " EZ_ANSI_GREEN "f|F : +|- zoom XYZ\n"
					EZ_ANSI_MAGENTA "     t  :   0:0:0  XYZ      " EZ_ANSI_GREEN " g  :    1:1:1 XYZ\n\n"

					  EZ_ANSI_WHITE "    /|? : show stats of frame set | current frame\n"
									"      z : show this information again\n\n"

									"                                X          Y          Z\n"
					 EZ_ANSI_YELLOW "current viewing angle  : "  << setw(8)
								 << ild.viewing_angle.x * one_radian << " , " << setw(8)
								 << ild.viewing_angle.y * one_radian << " , " << setw(8)
								 << ild.viewing_angle.z * one_radian << "" << endl <<
					  EZ_ANSI_GREEN "current viewing scale  : "  << setw(8)
								 << ild.viewing_scale.x << " , " << setw(8)
								 << ild.viewing_scale.y << " , " << setw(8)
								 << ild.viewing_scale.z << endl  <<
					EZ_ANSI_MAGENTA "current viewing offset : "   << setw(8)
								 << ild.viewing_offset.x << " , " << setw(8)
								 << ild.viewing_offset.y << " , " << setw(8)
								 << ild.viewing_offset.z << " points" << endl << endl <<
					  EZ_ANSI_WHITE "press and hold view parameters : "
					  EZ_ANSI_RESET
					  << endl;
							break;
				//----------------------------------------------------------------











				//----------------------------------------------------------------
				case ' ': // select, deselect current frame
					ild.current_frame().is_selected = !(ild.current_frame().is_selected);
					break;
				//----------------------------------------------------------------
				case '(': // set, unset start frame select
					ild.toggle_frame_select_start();
					break;
				//----------------------------------------------------------------
				case ')': // set, unset end frame select
					ild.toggle_frame_select_end();
					break;
				//----------------------------------------------------------------
				case '/': // show frame set stats
					cout << "\n\n\n\n\n\n";
					ild.tell();
					break;
				//----------------------------------------------------------------
				case '?': // show frame stats
					cout << "\n\n\n\n\n\n";
					ild.current_frame().stats(ild.show_cursors);
					break;
				//----------------------------------------------------------------











				//----------------------------------------------------------------
				case '`': // start animation forward
					while(    !ezfb_kb_hit()
						   && (    loop_animate
								|| (ild.frame_index < ild.number_of_frames() - 1)
							  )
						 )
					{
						time_zero = rdtsc();
						++ild;
						ild.display();
						if(approximate_frame_rate)
						{
							do{;}
							while(   rdtsc() - time_zero
								   < (unsigned long long int)
									 ((1.0 / ild.frames_per_second) * ticks_per_second) 
								 );
						}
					}
					break;
				//----------------------------------------------------------------
				case '~': // start animation reverse
					while(    !ezfb_kb_hit()
						   && (    loop_animate
								|| (ild.frame_index > 0)
							  )
						 )
					{
						time_zero = rdtsc();
						--ild;
						ild.display();
						if(approximate_frame_rate)
						{
							do{;}
							while(   rdtsc() - time_zero
								   < (unsigned long long int)
									 ((1.0 / ild.frames_per_second) * ticks_per_second) 
								 );
						}
					}
					break;
				//----------------------------------------------------------------














				//----------------------------------------------------------------
				case '-': // start rotation formard
					while(!ezfb_kb_hit())
					{
						time_zero = rdtsc();
						ild.viewing_angle += one_degree;
						ild.display();
						if(approximate_frame_rate)
						{
							do{;}
							while(   rdtsc() - time_zero
								   < (unsigned long long int)
									 ((1.0 / ild.frames_per_second) * ticks_per_second) 
								 );
						}
					}
					break;
				//----------------------------------------------------------------
				case '_': // start rotation reverse
					while(!ezfb_kb_hit())
					{
						time_zero = rdtsc();
						ild.viewing_angle -= one_degree;
						ild.display();
						if(approximate_frame_rate)
						{
							do{;}
							while(   rdtsc() - time_zero
								   < (unsigned long long int)
									 ((1.0 / ild.frames_per_second) * ticks_per_second) 
								 );
						}
					}
					break;
				//----------------------------------------------------------------




                


            
            
            
            
            
            
				//----------------------------------------------------------------
				case '=': // start animation with rotation forward
					while(!ezfb_kb_hit())
					{
						time_zero = rdtsc();
						ild.viewing_angle += one_degree;
						++ild;
						ild.display();
						if(approximate_frame_rate)
						{
							do{;}
							while(   rdtsc() - time_zero
								   < (unsigned long long int)
									 ((1.0 / ild.frames_per_second) * ticks_per_second) 
								 );
						}
					}
					break;
				//----------------------------------------------------------------
				case '+': // start animation with rotation reverse
					while(    !ezfb_kb_hit()
						   && (    loop_animate
								|| (ild.frame_index > 0)
							  )
						 )
					{
						time_zero = rdtsc();
						ild.viewing_angle -= one_degree;
						--ild;
						ild.display();
						if(approximate_frame_rate)
						{
							do{;}
							while(   rdtsc() - time_zero
								   < (unsigned long long int)
									 ((1.0 / ild.frames_per_second) * ticks_per_second) 
								 );
						}
					}
					break;
				//----------------------------------------------------------------




                


            
            
            
            
            
            
				//----------------------------------------------------------------
				case EZFB_CTRL_F: // open data from file
				{
					cout << EZ_ANSI_BOLD
							"\n\n\n\n\n\n"
			EZ_ANSI_RED     "    1: ilda\n"
			EZ_ANSI_YELLOW  "    2: dxf\n"
			EZ_ANSI_GREEN   "    3: wave\n"
			EZ_ANSI_CYAN    "    4: bitmap\n"
			EZ_ANSI_BLUE    "    5: ascii palette\n"
			EZ_ANSI_MAGENTA "[ESC]: cancel action\n"
			EZ_ANSI_WHITE   "select input file type :"
							EZ_ANSI_RESET
							<< endl;
					do
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '2'
							  && key != '3'
							  && key != '4'
							  && key != '5'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						break;
					}

					cout << (char)key << endl;

					switch(key)
					{
						//--------------------------------------------------------
						case '1': // open ilda                                
									cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
							EZ_ANSI_RED     "    1: replace current frame set\n"
							EZ_ANSI_YELLOW  "    2: prepend to current frame set\n"
							EZ_ANSI_GREEN   "    3: insert into current frame set after current frame\n"
							EZ_ANSI_CYAN   	"    4: append to current frame set\n"
							EZ_ANSI_BLUE    "    5: superimpose frame(s) into current frame set\n"
                            EZ_ANSI_MAGENTA "[ESC]: cancel action\n"
							EZ_ANSI_WHITE   "select input file opening option :"
											EZ_ANSI_RESET
										 << endl;
									do
									{
										key = ezfb_get_key();
									} while (    key != EZFB_ESCAPE_KEY
											  && key != '1'
											  && key != '2'
											  && key != '3'
											  && key != '4'
											  && key != '5'
											);

									if(key == EZFB_ESCAPE_KEY)
									{
										cout << "action cancelled" << endl << endl;
										break;
									}

									cout << (char)key << endl;
									list_ilda_files();
									switch(key)
									{
										//----------------------------------------
										case '1': // replace
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "opening ilda file ..." << endl;
													ild.from_file(input_string);
													break;
										//----------------------------------------
										case '2': // prepend
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "prepending ilda file ..." << endl;
													{
														ez_ilda_frame_set prepend;
														prepend.from_file(input_string);
														prepend += ild;
														ild = prepend;
													}                                                
													break;
										//----------------------------------------
										case '3': // insert
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "appending ilda file ..." << endl;
													{
														int                i;
														ez_ilda_frame_set  insert,
																		   result;
														insert.from_file(input_string);                                                    
														for(i = 0; i <= ild.frame_index; i++)
															result += ild[i];
														for(i = 0; i < insert.number_of_frames(); i++)
															result += insert[i];
														for(i = ild.frame_index + 1; i < ild.number_of_frames(); i++)
															result += ild[i];
														ild = result;
													}
													ild.from_file(input_string, true);
													break;
										//----------------------------------------
										case '4': // append
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "appending ilda file ..." << endl;
													ild.from_file(input_string, true);
													break;
										//----------------------------------------
										case '5': // superimpose
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "superimposing ilda file ..." << endl;
													ild.superimpose_from_file(input_string);
													break;
										//----------------------------------------
									}
									cout << input_string << " successfully processed" << endl;
									break;
						//--------------------------------------------------------
						case '2':
									cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
							EZ_ANSI_RED     "    1: replace current frame set with DXF file\n"
							EZ_ANSI_YELLOW  "    2: append  current frame set with DXF file\n"
							EZ_ANSI_GREEN   "    3: replace current frame set with DXF directory\n"
							EZ_ANSI_CYAN   	"    4: append  current frame set with DXF directory\n"
                            EZ_ANSI_BLUE    "[ESC]: cancel action\n"
							EZ_ANSI_WHITE   "select DXF input file opening option :"
											EZ_ANSI_RESET
										 << endl;
									do
									{
										key = ezfb_get_key();
									} while (    key != EZFB_ESCAPE_KEY
											  && key != '1'
											  && key != '2'
											  && key != '3'
											  && key != '4'
											);

									if(key == EZFB_ESCAPE_KEY)
									{
										cout << "action cancelled" << endl << endl;
										break;
									}

									cout << (char)key << endl;

									switch(key)
									{
										//----------------------------------------
										case '1': // open dxf
													list_dxf_files();
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "opening dxf file ..." << endl;
													ild.from_dxf_file(input_string);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
										case '2': // append dxf
													list_dxf_files();
													cout << "\nenter new file name : ";												
													ezfb_get_chars(input_string);
													cout << "appending dxf file ..." << endl;
													ild.from_dxf_file(input_string, true);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
										case '3': // open dxf directory
													list_dxf_directories();
													cout << "\nenter new directory name : ";
													ezfb_get_chars(input_string);
													cout << "opening dxf directory ..." << endl;
													ild.from_dxf_directory(input_string);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
										case '4': // append dxf directory
													list_dxf_directories();
													cout << "\nenter new directory name : ";
													ezfb_get_chars(input_string);
													cout << "appending dxf file ..." << endl;
													ild.from_dxf_directory(input_string, true);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
									}
									cout << ild.file_name << " now open" << endl;
									break;
						//--------------------------------------------------------
						case '3': // open wave                                
									list_wave_files();
									cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
							EZ_ANSI_RED     "    1: replace current frame set" << endl <<
							EZ_ANSI_YELLOW  "    2: append to current frame set"      << endl <<
                            EZ_ANSI_GREEN   "[ESC]: cancel action\n"
							EZ_ANSI_WHITE   "select wave file opening option :"
											EZ_ANSI_RESET
										 << endl;
									do
									{
										key = ezfb_get_key();
									} while (    key != EZFB_ESCAPE_KEY
											  && key != '1'
											  && key != '2'
											);

									if(key == EZFB_ESCAPE_KEY)
									{
										cout << "action cancelled" << endl << endl;
										break;
									}

									cout << (char)key << endl;

									switch(key)
									{
										//----------------------------------------
										case '1': // open wave
													list_wave_files();
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "opening wave file ..." << endl;
													ild.from_wave_file(input_string);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
										case '2': // append wave
													list_wave_files();
													cout << "\nenter new file name : ";
													ezfb_get_chars(input_string);
													cout << "appending wave file ..." << endl;
													ild.from_wave_file(input_string, true);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
									}
									cout << ild.file_name << " now open" << endl;
									break;
						//--------------------------------------------------------
						case '4': // open bitmap
									cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
							EZ_ANSI_RED     "    1: open as frame set color map\n"
							EZ_ANSI_YELLOW  "    2: open into frame background\n"
							EZ_ANSI_GREEN   "    3: clear frame background\n"
                            EZ_ANSI_CYAN    "[ESC]: cancel action\n"
							EZ_ANSI_WHITE   "select bitmap file opening option :"
											EZ_ANSI_RESET
										 << endl;
									do
									{
										key = ezfb_get_key();
									} while (    key != EZFB_ESCAPE_KEY
											  && key != '1'
											  && key != '2'
											  && key != '3'
											);

									if(key == EZFB_ESCAPE_KEY)
									{
										cout << "action cancelled" << endl << endl;
										break;
									}

									cout << (char)key << endl;

									switch(key)
									{
										//----------------------------------------
										case '1': //
													list_bmp_files();
													cout << "\nenter bmp file name : ";
													ezfb_get_chars(input_string);
													cout << "opening bmp file ..." << endl;
													ild.color_over_ezfb_bitmap(input_string);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
										case '2': //
													list_bmp_files();
													cout << "\nenter bmp file name : ";
													ezfb_get_chars(input_string);
													cout << "opening bmp file ..." << endl;
													ild.load_background_bitmap(input_string);
													cout << ild.file_name << " now open" << endl;
													break;
										//----------------------------------------
										case '3': //
													cout << "clearing background ..." << endl;
													ild.clear_background_bitmap();
													break;
										//----------------------------------------
									}                                
									break;
						//--------------------------------------------------------
						case '5': // ascii palette
									list_palette_files();
									cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "opening palette file ..." << endl;
									ild.add_palette_from_file(input_string);
									cout << input_string << " now open" << endl;                              
									break;
						//--------------------------------------------------------
					}
					show_hot_keys();
					break;
				}
				//----------------------------------------------------------------




				








				//----------------------------------------------------------------
				case EZFB_CTRL_O: // hide/show points, blanking, origin
				{
					if(   ild.show_points
						| ild.show_blanking
						| ild.show_palettes
						| ild.show_stats
						| ild.show_fixed_bounds
						| ild.show_fixed_origin
						| ild.show_floating_axis
						| ild.show_floating_bounds
					  )
						cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
							 << ((ild.show_points         ) ? (EZ_ANSI_RED     "points\n"         ) : (""))
							 << ((ild.show_blanking       ) ? (EZ_ANSI_YELLOW  "blanking\n"       ) : (""))
							 << ((ild.show_palettes       ) ? (EZ_ANSI_GREEN   "palettes\n"       ) : (""))
							 << ((ild.show_stats          ) ? (EZ_ANSI_CYAN    "stats\n"          ) : (""))
							 << ((ild.show_fixed_bounds   ) ? (EZ_ANSI_BLUE    "fixed bounds\n"   ) : (""))
							 << ((ild.show_fixed_origin   ) ? (EZ_ANSI_MAGENTA "fixed origin\n"   ) : (""))
							 << ((ild.show_floating_axis  ) ? (EZ_ANSI_RED     "floating axis\n"  ) : (""))
							 << ((ild.show_floating_bounds) ? (EZ_ANSI_YELLOW  "floating bounds\n") : (""))
							 << EZ_ANSI_RESET
							 << endl;
					else
						cout << EZ_ANSI_BOLD EZ_ANSI_WHITE "\n\n\n\n\n\nno display artifacts" EZ_ANSI_RESET << endl;

					cout << EZ_ANSI_BOLD
			EZ_ANSI_RED     "    1: " << ((ild.show_points         ) ? ("hide") : ("show")) << " points\n"
			EZ_ANSI_YELLOW  "    2: " << ((ild.show_blanking       ) ? ("hide") : ("show")) << " blanking\n"
			EZ_ANSI_GREEN   "    3: " << ((ild.show_palettes       ) ? ("hide") : ("show")) << " palettes\n"
			EZ_ANSI_CYAN    "    4: " << ((ild.show_stats          ) ? ("hide") : ("show")) << " stats\n"
			EZ_ANSI_BLUE    "    5: " << ((ild.show_fixed_bounds   ) ? ("hide") : ("show")) << " fixed bounds\n"
			EZ_ANSI_MAGENTA "    6: " << ((ild.show_fixed_origin   ) ? ("hide") : ("show")) << " fixed origin\n"
			EZ_ANSI_RED   	"    7: " << ((ild.show_floating_axis  ) ? ("hide") : ("show")) << " floating axis\n"
			EZ_ANSI_YELLOW  "    8: " << ((ild.show_floating_bounds) ? ("hide") : ("show")) << " floating bounds\n"
			EZ_ANSI_GREEN   "    9: " << "show all\n"
			EZ_ANSI_CYAN    "    0: " << "hide all\n"
			EZ_ANSI_BLUE    "[ESC]: cancel action\n"
			EZ_ANSI_WHITE   "select display artifacts :"
			EZ_ANSI_RESET   << endl;
					do
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '2'
							  && key != '3'
							  && key != '4'
							  && key != '5'
							  && key != '6'
							  && key != '7'
							  && key != '8'
							  && key != '9'
							  && key != '0'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						break;
					}

					cout << (char)key << endl;

					switch(key)
					{
						case '1':   ild.show_points          = !ild.show_points;
									break;
						//--------------------------------------------------------
						case '2':   ild.show_blanking        = !ild.show_blanking;
									break;
						//--------------------------------------------------------
						case '3':   ild.show_palettes        = !ild.show_palettes;
									break;
						//--------------------------------------------------------
						case '4':   ild.show_stats           = !ild.show_stats;
									break;
						//--------------------------------------------------------
						case '5':   ild.show_fixed_bounds    = !ild.show_fixed_bounds;
									break;
						//--------------------------------------------------------
						case '6':   ild.show_fixed_origin    = !ild.show_fixed_origin;
									break;
						//--------------------------------------------------------
						case '7':   ild.show_floating_axis   = !ild.show_floating_axis;
									break;
						//--------------------------------------------------------
						case '8':   ild.show_floating_bounds = !ild.show_floating_bounds;
									break;
						//--------------------------------------------------------
						case '9':   ild.show_points          = true;
									ild.show_blanking        = true;
									ild.show_palettes        = true;
									ild.show_stats           = true;
									ild.show_fixed_bounds    = true;
									ild.show_fixed_origin    = true;
									ild.show_floating_axis   = true;
									ild.show_floating_bounds = true;
									break;
						//--------------------------------------------------------
						case '0':   ild.show_points          = false;
									ild.show_blanking        = false;
									ild.show_palettes        = false;
									ild.show_stats           = false;
									ild.show_fixed_bounds    = false;
									ild.show_fixed_origin    = false;
									ild.show_floating_axis   = false;
									ild.show_floating_bounds = false;
									break;
						//--------------------------------------------------------
					}
					if(   ild.show_points
						| ild.show_blanking
						| ild.show_palettes
						| ild.show_stats
						| ild.show_fixed_bounds
						| ild.show_fixed_origin
						| ild.show_floating_axis
						| ild.show_floating_bounds
					  )
						cout << "\n" EZ_ANSI_BOLD
							 << ((ild.show_points         ) ? (EZ_ANSI_RED     "points\n"         ) : (""))
							 << ((ild.show_blanking       ) ? (EZ_ANSI_YELLOW  "blanking\n"       ) : (""))
							 << ((ild.show_palettes       ) ? (EZ_ANSI_GREEN   "palettes\n"       ) : (""))
							 << ((ild.show_stats          ) ? (EZ_ANSI_CYAN    "stats\n"          ) : (""))
							 << ((ild.show_fixed_bounds   ) ? (EZ_ANSI_BLUE    "fixed_bounds\n"   ) : (""))
							 << ((ild.show_fixed_origin   ) ? (EZ_ANSI_MAGENTA "fixed origin\n"   ) : (""))
							 << ((ild.show_floating_axis  ) ? (EZ_ANSI_RED     "floating axis\n"  ) : (""))
							 << ((ild.show_floating_bounds) ? (EZ_ANSI_YELLOW  "floating bounds\n") : (""))
							 << EZ_ANSI_RESET
							 << endl;
					else
						cout << EZ_ANSI_BOLD EZ_ANSI_WHITE "\nno display artifacts" EZ_ANSI_RESET << endl;
					cout << "Change observable display artifacts, DONE!" << endl;
					break;				
				}
				//----------------------------------------------------------------

                
                
                

            




				//----------------------------------------------------------------
				case EZFB_CTRL_V: // apply view to frame(s) data
				{
					ild.simplify_viewing_angle();
					cout << EZ_ANSI_BOLD EZ_ANSI_WHITE
							"\n\n\n\n\n\n"
							"                                X          Y          Z\n"
			 EZ_ANSI_YELLOW "current viewing angle  : "  << setw(8)
						 << ild.viewing_angle.x * one_radian << " , " << setw(8)
						 << ild.viewing_angle.y * one_radian << " , " << setw(8)
						 << ild.viewing_angle.z * one_radian << "" << endl <<
			  EZ_ANSI_GREEN "current viewing scale  : "  << setw(8)
						 << ild.viewing_scale.x << " , " << setw(8)
						 << ild.viewing_scale.y << " , " << setw(8)
						 << ild.viewing_scale.z << endl  <<
			EZ_ANSI_MAGENTA "current viewing offset : "   << setw(8)
						 << ild.viewing_offset.x << " , " << setw(8)
						 << ild.viewing_offset.y << " , " << setw(8)
						 << ild.viewing_offset.z << " points" << endl << endl <<
			 EZ_ANSI_YELLOW "    1: rotate frame to current viewing angle\n"
			  EZ_ANSI_GREEN "    2: zoom   frame to current viewing scale\n"
			EZ_ANSI_MAGENTA "    3: move   frame to current viewing offset\n"
							//"    4: apply all current viewing parameters to frame\n"
			  EZ_ANSI_RED   "[ESC]: cancel action\n"
			  EZ_ANSI_WHITE "select apply viewing option : \n"
							"[SHIFT] select to apply to all frames:"
							EZ_ANSI_RESET
				 		 << endl;
					do 
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '!'
							  && key != '2'
							  && key != '@'
							  && key != '3'
							  && key != '#'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						show_hot_keys();
						break;
					}

					cout << (char)key << endl;

					switch(key)
					{
						//--------------------------------------------------------
						case '1':
									ild.current_frame().rotate_around_origin();
									ild.viewing_angle *= one_degree;
									cout << " this frame set to angle : "
										 << ild.viewing_angle.x << ", "
										 << ild.viewing_angle.y << ", "
										 << ild.viewing_angle.z
										 << endl;
									ild.viewing_angle = 0.0;
									break;
						//--------------------------------------------------------
						case '!':
									ild.rotate_around_origin();
									ild.viewing_angle *= one_degree;
									cout << " all frames set to angle : "
										 << ild.viewing_angle.x << ", "
										 << ild.viewing_angle.y << ", "
										 << ild.viewing_angle.z
										 << endl;
									ild.viewing_angle = 0.0;
									break;
						//--------------------------------------------------------
						case '2':
									ild.current_frame().scale_around_origin();
									cout << " this frame set to scale : "
										 << ild.viewing_scale.x << ", "
										 << ild.viewing_scale.y << ", "
										 << ild.viewing_scale.z
										 << endl;
									ild.viewing_scale = 1.0;
									break;
						//--------------------------------------------------------
						case '@':                                
									ild.scale_around_origin();
									cout << " all frames set to scale : "
										 << ild.viewing_scale.x << ", "
										 << ild.viewing_scale.y << ", "
										 << ild.viewing_scale.z
										 << endl;
									ild.viewing_scale = 1.0;
									break;
						//--------------------------------------------------------
						case '3':
									ild.current_frame().move();
									cout << " this frame set to offset : "
										 << ild.viewing_offset.x << ", "
										 << ild.viewing_offset.y << ", "
										 << ild.viewing_offset.z
										 << endl;
									ild.viewing_offset = 0.0;                
									break;
						//--------------------------------------------------------
						case '#':
									ild.move();
									cout << " all frames set to offset : "
										 << ild.viewing_offset.x << ", "
										 << ild.viewing_offset.y << ", "
										 << ild.viewing_offset.z
										 << endl;
									ild.viewing_offset = 0.0;                
									break;
						//--------------------------------------------------------
						case '4':
									ild.current_frame().scale_around_origin();
									ild.current_frame().rotate_around_origin();
									ild.current_frame().move();

									cout << "current frame set to"
										 << endl
										 << "angle  : "                 << setw(8)
										 << ild.viewing_angle.x << ", " << setw(8)
										 << ild.viewing_angle.y << ", " << setw(8)
										 << ild.viewing_angle.z << endl
										 << "scale  : "                 << setw(8)
										 << ild.viewing_scale.x << ", " << setw(8)
										 << ild.viewing_scale.y << ", " << setw(8)
										 << ild.viewing_scale.z << endl
										 << "offset : "                  << setw(8)
										 << ild.viewing_offset.x << ", " << setw(8)
										 << ild.viewing_offset.y << ", " << setw(8)
										 << ild.viewing_offset.z << endl << endl;

									ild.viewing_angle  = 0.0;
									ild.viewing_scale  = 1.0;
									ild.viewing_offset = 0.0;
									break;
						//--------------------------------------------------------
						case '$':
									ild.scale_around_origin();
									ild.rotate_around_origin();
									ild.move();

									cout << "all frames set to"
										 << endl
										 << "angle  : "                  << setw(8)
										 << ild.viewing_angle.x << ", " << setw(8)
										 << ild.viewing_angle.y << ", " << setw(8)
										 << ild.viewing_angle.z << endl
										 << "scale  : "                  << setw(8)
										 << ild.viewing_scale.x << ", " << setw(8)
										 << ild.viewing_scale.y << ", " << setw(8)
										 << ild.viewing_scale.z << endl
										 << "offset : "                   << setw(8)
										 << ild.viewing_offset.x << ", " << setw(8)
										 << ild.viewing_offset.y << ", " << setw(8)
										 << ild.viewing_offset.z << endl  << endl;

									ild.viewing_angle  = 0.0;
									ild.viewing_scale  = 1.0;
									ild.viewing_offset = 0.0;
									break;
						//--------------------------------------------------------
					}
					cout << "Apply view to frame data, DONE!" << endl;
					break;
				}
				//----------------------------------------------------------------










                


                
	#if LASERBOY_DEMO
				//----------------------------------------------------------------
				case EZFB_CTRL_G: // save contents of memory to various file types
					cout << EZ_ANSI_BOLD
							"\n\n\n\n\n\n"
			EZ_ANSI_RED     "    1: save current frame as ilda\n"
							"    2: save selected frames as ilda\n"
							"    3: save all frames as ilda\n"
			EZ_ANSI_YELLOW  "    4: select and apply effect to current frame\n"
			EZ_ANSI_GREEN   "    5: select and apply effect to current frame set\n"
			EZ_ANSI_CYAN   	"    6: save current frame as dxf\n"
							"    7: save selected frames as directory of numbered dxf files\n"
							"    8: save all frames as directory of numbered dxf files\n"
			EZ_ANSI_BLUE    "    9: save current frame as wave still for " << ild.still_frame_duration << " seconds\n"
							"    0: save all frames as wave animation at " << ild.frames_per_second << " frames per second\n"
			EZ_ANSI_MAGENTA "    a: save current frame as compatible C++ code\n"
							"    b: save selected frames as compatible C++ code\n"
							"    c: save all frames as compatible C++ code\n"
			EZ_ANSI_RED     "    d: save current frame as " << ild.output_bmp_size <<  " pixel square bitmap\n"
							"    e: save selected frames as directory of numbered " << ild.output_bmp_size <<  " pixel square bitmaps\n"
							"    f: save all frames as directory of numbered " << ild.output_bmp_size <<  " pixel square bitmaps\n"
			EZ_ANSI_YELLOW  "    g: save current frame's palette as ascii table\n"
							"    h: save target palette as ascii table\n"
			EZ_ANSI_GREEN   "[ESC]: cancel action\n"
			EZ_ANSI_WHITE   "select file save option :\n\n"
							"DEMO does not allow these opperations."
							EZ_ANSI_RESET
							<< endl;
					break;
	#else
				//----------------------------------------------------------------
				case EZFB_CTRL_G: // save contents of memory to various file types
				{
					cout << EZ_ANSI_BOLD
							"\n\n\n\n\n\n"
			EZ_ANSI_RED     "    1: save current frame as ilda\n"
							"    2: save selected frames as ilda\n"
							"    3: save all frames as ilda\n"
			EZ_ANSI_YELLOW  "    4: select and apply effect to current frame\n"
			EZ_ANSI_GREEN   "    5: select and apply effect to current frame set\n"
			EZ_ANSI_CYAN   	"    6: save current frame as dxf\n"
							"    7: save selected frames as directory of numbered dxf files\n"
							"    8: save all frames as directory of numbered dxf files\n"
			EZ_ANSI_BLUE    "    9: save current frame as wave still for " << ild.still_frame_duration << " seconds\n"
							"    0: save all frames as wave animation at " << ild.frames_per_second << " frames per second\n"
			EZ_ANSI_MAGENTA "    a: save current frame as compatible C++ code\n"
							"    b: save selected frames as compatible C++ code\n"
							"    c: save all frames as compatible C++ code\n"
			EZ_ANSI_RED     "    d: save current frame as " << ild.output_bmp_size <<  " pixel square bitmap\n"
							"    e: save selected frames as directory of numbered " << ild.output_bmp_size <<  " pixel square bitmaps\n"
							"    f: save all frames as directory of numbered " << ild.output_bmp_size <<  " pixel square bitmaps\n"
			EZ_ANSI_YELLOW  "    g: save current frame's palette as ascii table\n"
							"    h: save target palette as ascii table\n"
			EZ_ANSI_GREEN   "[ESC]: cancel action\n"
			  EZ_ANSI_WHITE "select file save option :"
							EZ_ANSI_RESET
							<< endl;
					do
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '2'
							  && key != '3'
							  && key != '4'
							  && key != '5'
							  && key != '6'
							  && key != '7'
							  && key != '8'
							  && key != '9'
							  && key != '0'
							  && key != 'a'
							  && key != 'b'
							  && key != 'c'
							  && key != 'd'
							  && key != 'e'
							  && key != 'f'
							  && key != 'g'
							  && key != 'h'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						show_hot_keys();
						break;
					}

					cout << (char)key << endl;

					char input_string[81];

					switch(key)
					{
						//--------------------------------------------------------
						case '1':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving current frame as ilda" << endl;
									ild.current_frame().save_as_ilda(input_string);
									break;
						//--------------------------------------------------------
						case '2':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving selected frames as ilda" << endl;
									ild.selected_frames().save_as_ilda(input_string);
									break;
						//--------------------------------------------------------
						case '3':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving all frames as ilda" << endl;
									ild.save_as_ilda(input_string);
									break;
						//--------------------------------------------------------
						case '4':     ild.list_frame_effects();
									cout << "select and apply frame effect by name : " << endl;
									ezfb_get_chars(input_string);
									ild.call_frame_effect(input_string);
									break;
						//--------------------------------------------------------
						case '5':    ild.list_frame_set_effects();
									cout << "select and apply frame set effect by name : " << endl;
									ezfb_get_chars(input_string);
									ild.call_frame_set_effect(input_string);
									break;
						//--------------------------------------------------------
						case '6':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving current frame as dxf" << endl;
									ild.current_frame().save_as_dxf(input_string);
									break;
						//--------------------------------------------------------
						case '7':   cout << "\nenter new directory name : ";
									ezfb_get_chars(input_string);
									cout << "saving selected frames as dxf" << endl;
									ild.save_as_dxf_directory(input_string);
									break;
						//--------------------------------------------------------
						case '8':   cout << "\nenter new directory name : ";
									ezfb_get_chars(input_string);
									cout << "saving all frames as dxf" << endl;
									ild.selected_frames().save_as_dxf_directory(input_string);
									break;
						//--------------------------------------------------------
						case '9':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving current frame as still wave" << endl;
									ild.current_frame().save_as_wave(input_string);
									break;
						//--------------------------------------------------------
						case '0':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving all frames as wave animation" << endl;
									ild.save_as_wave(input_string);
									break;
						//--------------------------------------------------------
						case 'a':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving current frame as compatible C++ code" << endl;
									ild.current_frame().save_as_code(input_string);
									break;
						//--------------------------------------------------------
						case 'b':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving selected frames as compatible C++ code" << endl;
									ild.selected_frames().save_as_code(input_string);
									break;
						//--------------------------------------------------------
						case 'c':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving all frames as compatible C++ code" << endl;
									ild.save_as_code(input_string);
									break;
						//--------------------------------------------------------
						case 'd':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving current frame as bitmap image" << endl;
									ild.current_frame().save_as_bmp(input_string);
									break;
						//--------------------------------------------------------
						case 'e':   cout << "\nenter new directory name : ";
									ezfb_get_chars(input_string);
									cout << "saving selected frames as numbered bitmap images" << endl;
									ild.selected_frames().save_as_bmp_directory(input_string);
									break;
						//--------------------------------------------------------
						case 'f':   cout << "\nenter new directory name : ";
									ezfb_get_chars(input_string);
									cout << "saving all frames as numbered bitmap images" << endl;
									ild.save_as_bmp_directory(input_string);
									break;
						//--------------------------------------------------------
						case 'g':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving current frame's palette as ascii table" << endl;
									ild.palette_picker(ild.current_frame().palette_index).save_as_ascii(input_string);
									break;
						//--------------------------------------------------------
						case 'h':   cout << "\nenter new file name : ";
									ezfb_get_chars(input_string);
									cout << "saving target palette as ascii table" << endl;
									ild.palette_picker(ild.target_palette_index).save_as_ascii(input_string);
									break;
						//--------------------------------------------------------
					}
					cout << "saving data DONE!" << endl;					
					break;				
				}
				//----------------------------------------------------------------
	#endif













				//----------------------------------------------------------------
				case EZFB_TAB_KEY:
				{
					cout << EZ_ANSI_BOLD
							"\n\n\n\n\n\n"
			EZ_ANSI_RED     "    1: zero angle tolorance in degrees ........: " << (ild.zero_angle_tolerance * one_radian) << endl <<
			EZ_ANSI_YELLOW  "    2: distance delta max in points ...........: " << ild.distance_delta_max      << endl <<
			EZ_ANSI_GREEN   "    3: 180 corner dwell in microseconds ......: " << ild.longest_dwell_microsec  << endl <<
			EZ_ANSI_CYAN   	"    4: insignificant distance in points .......: " << ild.insignificant_distance  << endl <<
			EZ_ANSI_BLUE    "    5: rendered arc segment angle in degrees ..: " << ild.rendered_arc_angle      << endl <<
			EZ_ANSI_MAGENTA "    6: move points per key hit ................: " << move_points_per_key         << endl <<
			EZ_ANSI_RED     "    7: recolor palette span factor ............: " << recolor_span_factor         << endl <<
			EZ_ANSI_YELLOW  "    8: animation frames per second ............: " << ild.frames_per_second       << endl <<
			EZ_ANSI_GREEN   "    9: still frame durration in seconds .......: " << ild.still_frame_duration    << endl <<
			EZ_ANSI_CYAN   	"    0: output wave sample rate ................: " << ild.wave_sample_rate        << endl <<
			EZ_ANSI_BLUE    "    a: display font size factor in pixels .....: " << ild.font_size_factor        << endl <<
			EZ_ANSI_MAGENTA "    b: output bitmap size in square pixels ....: " << ild.output_bmp_size         << endl <<
			EZ_ANSI_RED     "    c: dwell points are .......................: " << ((ild.black_dwell_points)      ? ("black") : ("lit")) << endl <<
			EZ_ANSI_YELLOW  "    d: show live frame set rendering ..........: " << ((ild.show_effects_generation) ? ("on")    : ("off")) << endl <<
			EZ_ANSI_GREEN   "    e: flatten Z for wave output ..............: " << ((ild.auto_flatten_z)          ? ("on")    : ("off")) << endl <<
			EZ_ANSI_CYAN   	"    f: auto optimize before output ............: " << ((ild.auto_optimize)           ? ("on")    : ("off")) << endl <<
			EZ_ANSI_BLUE    "    g: loop animation .........................: " << ((loop_animate)                ? ("on")    : ("off")) << endl <<
			EZ_ANSI_MAGENTA "    h: animate at approximated frame rate .....: " << ((approximate_frame_rate)      ? ("on")    : ("off")) << endl <<
			EZ_ANSI_RED     "    i: auto scale dxf import ..................: " << ((ild.auto_scale_dxf)          ? ("on")    : ("off")) << endl <<
			EZ_ANSI_YELLOW  "    j: maintain dxf origin ....................: " << (   (ild.auto_scale_dxf)
																						? (   (ild.maintain_dxf_origin)
																							? ("on")
																							: ("off")
																						  )
																						: ("depends on auto scale dxf")
																					  )
																				 << endl <<
			EZ_ANSI_GREEN   "[ESC]: cancel action\n"
			EZ_ANSI_WHITE   "select system setting to change :"
			EZ_ANSI_RESET	<< endl;
					do
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '2'
							  && key != '3'
							  && key != '4'
							  && key != '5'
							  && key != '6'
							  && key != '7'
							  && key != '8'
							  && key != '9'
							  && key != '0'
							  && key != 'a'
							  && key != 'b'
							  && key != 'c'
							  && key != 'd'
							  && key != 'e'
							  && key != 'f'
							  && key != 'g'
							  && key != 'h'
							  && key != 'i'
							  && key != 'j'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						show_hot_keys();
						break;
					}

					cout << (char)key << endl;

					switch(key)
					{
						//--------------------------------------------------------
						case '1':   cout << "enter new zero angle tolerance in degrees : ";
									ild.zero_angle_tolerance = (ezfb_get_float() * one_degree);
									cout << "new zero angle tolerance : " << (ild.zero_angle_tolerance * one_radian) << endl;
									break;
						//--------------------------------------------------------
						case '2':   cout << "enter new distance delta max in points : ";
									ild.distance_delta_max = ezfb_get_float();
									cout << "new distance delta max : " << ild.distance_delta_max << endl;
									break;
						//--------------------------------------------------------
						case '3':   cout << "enter dwell in microseconds for 180 angle : ";
									ild.longest_dwell_microsec = ezfb_get_float();
									cout << "new dwell in microseconds for 180 angle : " << ild.longest_dwell_microsec << endl;
									break;
						//--------------------------------------------------------
						case '4':   cout << "enter insignificant distance in points : ";
									ild.insignificant_distance = ezfb_get_float();
									cout << "new insignificant distance : " << ild.insignificant_distance << endl;
									break;
						//--------------------------------------------------------
						case '5':   cout << "rendered arc segment angle in degrees : ";
									ild.rendered_arc_angle = ezfb_get_float();
									cout << "new rendered arc angle : " << ild.rendered_arc_angle << endl;
									break;
						//--------------------------------------------------------
						case '6':   cout << "enter new move points per key : ";
									move_points_per_key = ezfb_get_int();
									cout << "new move points per key  : " << move_points_per_key << endl;
									break;
						//--------------------------------------------------------
						case '7':   cout << "enter new recolor palette span factor : ";
									recolor_span_factor = ezfb_get_float();
									cout << "new recolor palette span factor  : " << recolor_span_factor << endl;
									break;
						//--------------------------------------------------------
						case '8':   cout << "enter new frames per second wave output : ";
									ild.frames_per_second = ezfb_get_float();
									cout << "new frames per second wave output  : " << ild.frames_per_second << endl;
									break;
						//--------------------------------------------------------
						case '9':   cout << "enter new durration for single frame wave output : ";
									ild.still_frame_duration = ezfb_get_float();
									cout << "new durration for single frame wave output  : " << ild.still_frame_duration << endl;
									break;
						//--------------------------------------------------------
						case '0':   cout << "enter new output wave sample rate : ";
									ild.wave_sample_rate = ezfb_get_int();
									cout << "new output wave sample rate : " << ild.wave_sample_rate << endl;
									break;
						//--------------------------------------------------------
						case 'a':   cout << "display fonst size factor : ";
									ild.font_size_factor = ezfb_get_int();
									cout << "new display fonst size factor : " << ild.font_size_factor << endl;
									break;
						//--------------------------------------------------------
						case 'b':   cout << "output bitmap size in pixels (square): ";
									ild.output_bmp_size = ezfb_get_int();
									cout << "new output bitmap size in pixels (square) : " << ild.output_bmp_size << endl;
									break;
						//--------------------------------------------------------
						case 'c':   cout << "toggle lit or black dwell points : ";
									ild.black_dwell_points = !ild.black_dwell_points;
									cout << "now dwell points will be : " << ((ild.black_dwell_points) ? ("black") : ("lit")) << endl;
									break;
						//--------------------------------------------------------
						case 'd':   cout << "show effects generation : " << endl;
									ild.show_effects_generation = !ild.show_effects_generation;
									cout << "now show effects generation will be : " << ((ild.show_effects_generation) ? ("on") : ("off")) << endl;
									break;
						//--------------------------------------------------------
						case 'e':   cout << "toggle flatten z for wave output : " << endl;
									ild.auto_flatten_z = !ild.auto_flatten_z;
									cout << "now flatten z for wave output will be : " << ((ild.auto_flatten_z) ? ("on") : ("off")) << endl;
									break;
						//--------------------------------------------------------
						case 'f':   cout << "toggle auto optimize for wave output : " << endl;
									ild.auto_optimize = !ild.auto_optimize;
									cout << "now auto optimize for wave output will be : " << ((ild.auto_optimize) ? ("on") : ("off")) << endl;
									break;
						//--------------------------------------------------------
						case 'g':   cout << "toggle loop animation on arrow keys : " << endl;
									loop_animate = !loop_animate;
									cout << "now loop animation on arrow keys will be : " << ((loop_animate) ? ("on") : ("off")) << endl;
									break;
						//--------------------------------------------------------
						case 'h':   cout << "toggle animate at approximated frame rate : " << endl;
									approximate_frame_rate = !approximate_frame_rate;
									cout << "now animate at approximated frame rate will be : " << ((approximate_frame_rate) ? ("on") : ("off")) << endl;
									break;
						//--------------------------------------------------------
						case 'i':   cout << "toggle auto scale dxf import : " << endl;
									ild.auto_scale_dxf = !ild.auto_scale_dxf;
									cout << "now auto scale dxf import will be : " << ((ild.auto_scale_dxf) ? ("on") : ("off")) << endl;
									break;
						//--------------------------------------------------------
						case 'j':   if(ild.auto_scale_dxf)
									{
										cout << "toggle maintain dxf origin : " << endl;
										ild.maintain_dxf_origin = !ild.maintain_dxf_origin;
										cout << "now maintain dxf origin will be : " << ((ild.maintain_dxf_origin) ? ("on") : ("off")) << endl;
									}
									else
										cout << "auto scale dxf must be on" << endl;
									break;
						//--------------------------------------------------------
					}
					cout << "Systems settings change, DONE!" << endl;
					break;
				}
				//----------------------------------------------------------------












				//----------------------------------------------------------------
				case EZFB_ARROW_UP: // display next palette
					ild.target_palette_index++;
					if(ild.target_palette_index >= ild.number_of_palettes())
						ild.target_palette_index = 0;
					break;
				//----------------------------------------------------------------
				case EZFB_ARROW_DOWN: // display previous palette
					ild.target_palette_index--;
					if(ild.target_palette_index < 0)
						ild.target_palette_index = ild.number_of_palettes() - 1;
					break;
				//----------------------------------------------------------------

                
                
                
                
                
                
                
                
                
                
                
				//----------------------------------------------------------------
				case EZFB_ARROW_RIGHT: // display previous frame
					if(    loop_animate
						|| (ild.frame_index < ild.number_of_frames() - 1)
					  )
						++ild;
					break;
				//----------------------------------------------------------------
				case EZFB_ARROW_LEFT: // display next frame
					if(    loop_animate
						|| (ild.frame_index > 0)
					  )
						--ild;
					break;
				//----------------------------------------------------------------


                                                               
                




                



                
                
				//----------------------------------------------------------------
				case EZFB_CTRL_B: // go to first frame
									cout << "first frame" << endl;
									ild.frame(0);
									break;
				//----------------------------------------------------------------
				case EZFB_CTRL_E: // go to last frame
									cout << "last frame" << endl;
									ild.frame(ild.number_of_frames() - 1);
									break;
				//----------------------------------------------------------------
				case EZFB_DELETE_KEY:
									cout << "delete current frame" << endl;
									ild.delete_current_frame();
									break;
				//---------------------------------------------------------------- 















                      
                      
				//----------------------------------------------------------------
				case EZFB_CTRL_P:
				{
					cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
			EZ_ANSI_RED     "    1: span x with palette in frame\n"
			EZ_ANSI_YELLOW  "    2: span y with palette in frame\n"
			EZ_ANSI_GREEN   "    3: span z with palette in frame\n"
			EZ_ANSI_CYAN   	"    4: span points with palette in frame\n"
			EZ_ANSI_BLUE    "    5: index points with palette in frame\n"
			EZ_ANSI_MAGENTA "    6: span lit segments with palette in frame\n"
			EZ_ANSI_RED     "    7: index lit segments with palette in frame\n"
			EZ_ANSI_YELLOW  "    8: random color points with palette in frame\n"
			EZ_ANSI_GREEN   "    9: random color lit segments with palette in frame\n"
			EZ_ANSI_CYAN   	"    0: set frame to target palette (by index)\n"
			EZ_ANSI_BLUE    "    a: best match frame to target palette\n"
			EZ_ANSI_MAGENTA "    b: convert true color frame to palette (if possible)\n"
			EZ_ANSI_RED     "    c: bit reduce true color frame to palette\n"
			EZ_ANSI_YELLOW  "    d: promote frame to true color\n"
			EZ_ANSI_GREEN   "[ESC]: cancel action\n"
			EZ_ANSI_WHITE   "select palette change option :\n"
							"[SHIFT] select to apply to all frames :"
			EZ_ANSI_RESET   << endl;
					do
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '!'
							  && key != '2'
							  && key != '@'
							  && key != '3'
							  && key != '#'
							  && key != '4'
							  && key != '$'
							  && key != '5'
							  && key != '%'
							  && key != '6'
							  && key != '^'
							  && key != '7'
							  && key != '&'
							  && key != '8'
							  && key != '*'
							  && key != '9'
							  && key != '('
							  && key != '0'
							  && key != ')'
							  && key != 'a'
							  && key != 'A'
							  && key != 'b'
							  && key != 'B'
							  && key != 'c'
							  && key != 'C'
							  && key != 'd'
							  && key != 'D'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						show_hot_keys();
						break;
					}

					cout << (char)key << endl;

					switch(key)
					{
						case '1':
									ild.current_frame().rainbow_recolor(0, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '!':
									ild.rainbow_recolor(0, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '2':
									ild.current_frame().rainbow_recolor(1, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '@':
									ild.rainbow_recolor(1, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '3':
									ild.current_frame().rainbow_recolor(2, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '#':
									ild.rainbow_recolor(2, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '4':
									ild.current_frame().rainbow_recolor(3, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '$':
									ild.rainbow_recolor(3, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '5':
									ild.current_frame().rainbow_recolor(4, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '%':
									ild.rainbow_recolor(4, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '6':
									ild.current_frame().rainbow_recolor(5, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '^':
									ild.rainbow_recolor(5, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '7':
									ild.current_frame().rainbow_recolor(6, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '&':
									ild.rainbow_recolor(6, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '8':
									ild.current_frame().rainbow_recolor(7, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '*':
									ild.rainbow_recolor(7, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '9':
									ild.current_frame().rainbow_recolor(8, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '(':
									ild.rainbow_recolor(8, recolor_span_factor);
									break;
						//--------------------------------------------------------
						case '0':   ild.current_frame().accept_target_palette();
									cout << "accept target palette by index" << endl;
									break;
						//--------------------------------------------------------
						case ')':   ild.accept_target_palette();
									cout << "accept target palette by index" << endl;
									break;
						//--------------------------------------------------------
						case 'a':   ild.current_frame().best_match_target_palette();
									cout << "best match to target palette" << endl;
									break;
						//--------------------------------------------------------
						case 'A':   ild.best_match_target_palette();
									cout << "best match to target palette" << endl;
									break;
						//--------------------------------------------------------
						case 'b':   ild.current_frame().sync_rgb_and_palette();
									cout << "convert to palette" << endl;
									break;
						//--------------------------------------------------------
						case 'B':   ild.sync_rgb_and_palette();
									cout << "convert to palette" << endl;
									break;
						//--------------------------------------------------------
						case 'c':   ild.current_frame().bit_reduce_to_palette();
									cout << "bit reduce true color to palette" << endl;
									break;
						//--------------------------------------------------------
						case 'C':   ild.bit_reduce_to_palette();
									cout << "bit reduce true color to palette" << endl;
									break;
						//--------------------------------------------------------
						case 'd':   ild.current_frame().promote_to_true_color();
									cout << "promote to true color" << endl;
									break;
						//--------------------------------------------------------
						case 'D':   ild.promote_to_true_color();
									cout << "promote to true color" << endl;
									break;
						//--------------------------------------------------------
					}
					cout << "Change palette effect, DONE!" << endl;
					break;
				}
				//----------------------------------------------------------------




            
































                        
				//----------------------------------------------------------------
				case EZFB_CTRL_D: // enter points color and blanking editor
				{
					char    font[81];
					int     number_of_sides;
					float   x_rotation,
							arc_angle,
							radii_ratio;
                
					ild.show_cursors = true;
					ild.display();
						 // "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
					cout << EZ_ANSI_BOLD
							"\n\n\n\n\n\n"
			EZ_ANSI_WHITE   "draw, move, recolor points and segments mode activated\n"
			EZ_ANSI_RED     "      \\||       : select-start to first | select-end to last\n"
							"[|{    |    ]|} : select start | end        forward | backward one point\n"
							"i|I | o|O | u|U : select start | end | both forward | backward per lit segment\n"
			EZ_ANSI_YELLOW  "       a        : unblank all points in frame\n"
							"      b|B       : blank | unblank selected-points in frame\n"
			EZ_ANSI_GREEN   "      p|P       : +|- selected palette color index\n"
							"       c        : color selected-points with selected palette color index\n"
							"       C        : set palette color index to select-start color\n"
			EZ_ANSI_CYAN    "      m|M       : +|-1% magnitude of select-start\n"
							"f|F    |    g|G : rotate select-start +|-1 around Z|Y axis\n"
							"x|X | y|Y | z|Z : move select-start X|Y|Z +|-" << move_points_per_key <<
																			 " points\n"
							" q  |  w  |  e  : set  select-start X|Y|Z coordinate\n"
			EZ_ANSI_BLUE    "       l        : draw a line from select-start to select-end\n"
							"       t        : render text on baseline of select-start vector\n"
							"      d|D       : draw " << ild.rendered_arc_angle <<
													   " circular | eliptical arc on select-start vector\n"
							"     CTRL-D     : draw polygon on select-start vector\n"
			EZ_ANSI_MAGENTA "       =        : add new point after select-start\n"
							"       s        : insert a blank point at select-start (break segment)\n"
							"  [BACKSPACE]   : remove point at select-start\n"
			EZ_ANSI_RED     "      /|?       : show stats of frame set | current frame\n"
			EZ_ANSI_YELLOW  "     [ESC]      : exit draw, move, recolor points and segments mode\n"
			EZ_ANSI_WHITE   "  ARROW_KEYs and view rotation are active"
			EZ_ANSI_RESET   << endl;

					while((key = ezfb_get_key()) != EZFB_ESCAPE_KEY) // from ezfb_tty.c
					{   // ezfb_get_key() accepts a single key stroke with no return
						switch(key)
						{
							//----------------------------------------------------
							case '/': // show frame set stats
										cout << "\n\n\n\n\n\n";
										ild.tell();
										break;
							//----------------------------------------------------
							case '?': // show frame stats
										cout << "\n\n\n\n\n\n";
										ild.current_frame().stats(ild.show_cursors);
										break;
							//----------------------------------------------------
							case '=':   ild.current_frame().add_new_point(ild.current_frame().point_select_start);
										break;
							//----------------------------------------------------
							case '[':   ild.current_frame().increment_point_select_start();
										break;
							case '{':   ild.current_frame().deccrement_point_select_start();
										break;
							case'\\':   ild.current_frame().point_select_start = 0;
										cout << "first point" << endl;
										break;
							//----------------------------------------------------
							case ']':   ild.current_frame().increment_point_select_end();
										break;
							case '}':   ild.current_frame().deccrement_point_select_end();
										break;
							case '|':   ild.current_frame().point_select_end = (ild.current_frame().number_of_points() - 1);
										cout << "last point" << endl;
										break;
							//----------------------------------------------------
							case 'u':   ild.current_frame().next_segment_select();
										//cout << "next lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'U':   ild.current_frame().previous_segment_select();
										//cout << "previous lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'i':   ild.current_frame().next_segment_select_start();
										//cout << "next lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'I':   ild.current_frame().previous_segment_select_start();
										//cout << "previous lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'o':   ild.current_frame().next_segment_select_end();
										//cout << "next lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'O':   ild.current_frame().previous_segment_select_end();
										//cout << "previous lit segment" << endl;
										break;
							//----------------------------------------------------            
							case 'q':	ild.current_frame().at(ild.current_frame().point_select_start).tell();
										cout << "enter new value for X : ";
										ild.current_frame().at(ild.current_frame().point_select_start).x = ezfb_get_int();
										cout << endl;
										ild.current_frame().at(ild.current_frame().point_select_start).tell();
										break;
							//----------------------------------------------------            
							case 'w':  	ild.current_frame().at(ild.current_frame().point_select_start).tell();
										cout << "enter new value for Y : ";
										ild.current_frame().at(ild.current_frame().point_select_start).y = ezfb_get_int();
										cout << endl;
										ild.current_frame().at(ild.current_frame().point_select_start).tell();
										break;
							//----------------------------------------------------            
							case 'e':  	ild.current_frame().at(ild.current_frame().point_select_start).tell();
										cout << "enter new value for Z : ";
										ild.current_frame().at(ild.current_frame().point_select_start).z = ezfb_get_int();
										cout << endl;
										ild.current_frame().at(ild.current_frame().point_select_start).tell();
										break;
							//----------------------------------------------------
							case 'a':   ild.current_frame().point_select_end++;
										cout << "blank all" << endl;
										break;
							//----------------------------------------------------                        
							case 'b':   ild.current_frame().blank_points();
										cout << "blank selected-points" << endl;
										break;
							//----------------------------------------------------
							case 'B':   ild.current_frame().unblank_points();
										cout << "unblank selected-points" << endl;
										break;
							//----------------------------------------------------                        
							case 'c':   ild.current_frame().set_points_color_index();
										cout << "color selected-points" << endl;
										break;
							//----------------------------------------------------                        
							case 'C':   ild.selected_palette_color = ild.current_frame().at(ild.current_frame().point_select_start).c;
										cout << "pick color at select-start" << endl;
										break;
							//----------------------------------------------------            
							case 't':   if(ild.current_frame().point_select_start) // is not the origin
										{
											list_ilda_files();
											cout << "enter font (ilda) file name" << endl;
											ezfb_get_chars(font);

											cout << "enter angle of rotation around vector" << endl;
											x_rotation = ezfb_get_float();

											cout << "enter upto 80 characters of text" << endl;
											ezfb_get_line(input_string, CHARS_IN_LINE);
											//ezfb_get_chars(input_string);

											ild.current_frame()
											+= ez_ilda_segment(	  ild.current_frame().at(ild.current_frame().point_select_start - 1)
																, ild.current_frame().at(ild.current_frame().point_select_start)
																, x_rotation
																, string(font)
																, string(input_string)
															  );

											ild.current_frame().at(ild.current_frame().point_select_start).blank();
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------            
							case EZFB_CTRL_D:
							            if(ild.current_frame().point_select_start) // is not the origin
										{
											cout << "enter integer number of sides" << endl;
											number_of_sides = ezfb_get_int();
											ild.current_frame()
											+= ez_ilda_segment(	  ild.current_frame().at(ild.current_frame().point_select_start - 1)
																, ild.current_frame().at(ild.current_frame().point_select_start)
																, number_of_sides
															  );
											ild.current_frame().at(ild.current_frame().point_select_start).blank();
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------            
							case 'd':   if(ild.current_frame().point_select_start)
										{
											cout << "enter arc angle in degrees, (0 = 360)" << endl;
											arc_angle = ezfb_get_float();
											ild.current_frame()
											+= ez_ilda_segment(	  ild.current_frame().at(ild.current_frame().point_select_start - 1)
																, ild.current_frame().at(ild.current_frame().point_select_start)
																, arc_angle
															  );
											ild.current_frame().at(ild.current_frame().point_select_start).blank();
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------            
							case 'D':		if(ild.current_frame().point_select_start)
										{
											cout << "enter arc angle in degrees, (0 = 360)" << endl;
											arc_angle = ezfb_get_float();
											cout << "enter minor / major radii ratio" << endl;
											radii_ratio = ezfb_get_float();
											ild.current_frame()
											+= ez_ilda_segment(	  ild.current_frame().at(ild.current_frame().point_select_start - 1)
																, ild.current_frame().at(ild.current_frame().point_select_start)
																, arc_angle
																, radii_ratio
															  );
											ild.current_frame().at(ild.current_frame().point_select_start).blank();
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------            
							case 'l':  	ild.current_frame().connect_the_dots();
										break;
							//----------------------------------------------------            
							case 's':	ild.current_frame().break_segment();
										break;
							//----------------------------------------------------
							case 'p':   ild.selected_palette_color++;
										if(ild.selected_palette_color >= ild.current_palette().number_of_colors())
											ild.selected_palette_color = 0;
										break;
							//----------------------------------------------------
							case 'P':   ild.selected_palette_color--;
										if(ild.selected_palette_color <= 0)
											ild.selected_palette_color = ild.current_palette().number_of_colors() - 1;
										break;
							//----------------------------------------------------
							case 'x':   temp_float = ild.current_frame().at(ild.current_frame().point_select_start).x + move_points_per_key;
										if(temp_float <= EZ_MAX_SHORT)
											ild.current_frame().at(ild.current_frame().point_select_start).x = (u_short)temp_float;
										break;
							//----------------------------------------------------
							case 'X':   temp_float = ild.current_frame().at(ild.current_frame().point_select_start).x - move_points_per_key;
										if(temp_float >= EZ_MIN_SHORT)
											ild.current_frame().at(ild.current_frame().point_select_start).x = (u_short)temp_float;
										break;
							//----------------------------------------------------
							case 'y':   temp_float = ild.current_frame().at(ild.current_frame().point_select_start).y + move_points_per_key;
										if(temp_float <= EZ_MAX_SHORT)
											ild.current_frame().at(ild.current_frame().point_select_start).y = (u_short)temp_float;
										break;
							//----------------------------------------------------
							case 'Y':   temp_float = ild.current_frame().at(ild.current_frame().point_select_start).y - move_points_per_key;
										if(temp_float >= EZ_MIN_SHORT)
											ild.current_frame().at(ild.current_frame().point_select_start).y = (u_short)temp_float;
										break;
							//----------------------------------------------------
							case 'z':   temp_float = ild.current_frame().at(ild.current_frame().point_select_start).z + move_points_per_key;
										if(temp_float <= EZ_MAX_SHORT)
											ild.current_frame().at(ild.current_frame().point_select_start).z = (u_short)temp_float;
										break;
							//----------------------------------------------------
							case 'Z':   temp_float = ild.current_frame().at(ild.current_frame().point_select_start).z - move_points_per_key;
										if(temp_float >= EZ_MIN_SHORT)
											ild.current_frame().at(ild.current_frame().point_select_start).z = (u_short)temp_float;
										break;
							//----------------------------------------------------
							case 'f':   if(ild.current_frame().point_select_start) // is not the origin
										{
											temp_3D_float
											= rotate_point_around_point_z(   ild.current_frame().at(ild.current_frame().point_select_start)
																		   , ild.current_frame().at(ild.current_frame().point_select_start - 1)
																		   , one_degree
																		 );
											if(!ez_ilda_bounds_check(temp_3D_float, EZ_ILDA_CUBE))
												ild.current_frame().at(ild.current_frame().point_select_start) = temp_3D_float;
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------
							case 'F':   if(ild.current_frame().point_select_start) // is not the origin
										{
											temp_3D_float
											= rotate_point_around_point_z(   ild.current_frame().at(ild.current_frame().point_select_start)
																		   , ild.current_frame().at(ild.current_frame().point_select_start - 1)
																		   , -one_degree
																		 );
											if(!ez_ilda_bounds_check(temp_3D_float, EZ_ILDA_CUBE))
												ild.current_frame().at(ild.current_frame().point_select_start) = temp_3D_float;
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------
							case 'g':   if(ild.current_frame().point_select_start) // is not the origin
										{
											temp_3D_float
											= rotate_point_around_point_y(   ild.current_frame().at(ild.current_frame().point_select_start)
																		   , ild.current_frame().at(ild.current_frame().point_select_start - 1)
																		   , one_degree
																		 );
											if(!ez_ilda_bounds_check(temp_3D_float, EZ_ILDA_CUBE))
												ild.current_frame().at(ild.current_frame().point_select_start) = temp_3D_float;

										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------
							case 'G':   if(ild.current_frame().point_select_start) // is not the origin
										{
											temp_3D_float
											= rotate_point_around_point_y(   ild.current_frame().at(ild.current_frame().point_select_start)
																		   , ild.current_frame().at(ild.current_frame().point_select_start - 1)
																		   , -one_degree
																		 );
											if(!ez_ilda_bounds_check(temp_3D_float, EZ_ILDA_CUBE))
												ild.current_frame().at(ild.current_frame().point_select_start) = temp_3D_float;
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------
							case 'm':   if(ild.current_frame().point_select_start) // is not the origin
										{
											temp_3D_float
											= scale_point_around_point(   ild.current_frame().at(ild.current_frame().point_select_start)
																		, ild.current_frame().at(ild.current_frame().point_select_start - 1)
																		, ez_3D_float(1.01, 1.01, 1.01)
																	  );
											if(!ez_ilda_bounds_check(temp_3D_float, EZ_ILDA_CUBE))
												ild.current_frame().at(ild.current_frame().point_select_start) = temp_3D_float;
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------
							case 'M':   if(ild.current_frame().point_select_start) // is not the origin
										{

											temp_3D_float                                          
											= scale_point_around_point(   ild.current_frame().at(ild.current_frame().point_select_start)
																		, ild.current_frame().at(ild.current_frame().point_select_start - 1)
																		, ez_3D_float(0.99, 0.99, 0.99)
																	  );
											if(!ez_ilda_bounds_check(temp_3D_float, EZ_ILDA_CUBE))
												ild.current_frame().at(ild.current_frame().point_select_start) = temp_3D_float;
										}
										else
											cout << "not valid at the zeroeth point!" << endl;
										break;
							//----------------------------------------------------
							case EZFB_BACKSPACE_KEY:
										ild.current_frame().remove_point(ild.current_frame().point_select_start);
										break;
							//----------------------------------------------------
							case EZFB_ARROW_UP: // display next palette
										ild.target_palette_index++;
										if(ild.target_palette_index >= ild.number_of_palettes())
											ild.target_palette_index = 0;
										break;
							//----------------------------------------------------
							case EZFB_ARROW_DOWN: // display previous palette
										ild.target_palette_index--;
										if(ild.target_palette_index <= 0)
											ild.target_palette_index = ild.number_of_palettes() - 1;
										break;
							//----------------------------------------------------
							case EZFB_ARROW_RIGHT: // display previous frame
										if(    loop_animate
											|| (ild.frame_index < ild.number_of_frames() - 1)
										  )
											++ild;
										break;
							//----------------------------------------------------
							case EZFB_ARROW_LEFT: // display next frame
										if(    loop_animate
											|| (ild.frame_index > 0)
										  )
											--ild;
										break;
							//----------------------------------------------------
							case '1':   ild.viewing_angle.x += one_degree;
										break;
							//----------------------------------------------------
							case '!':   ild.viewing_angle.x -= one_degree;
										break;
							//----------------------------------------------------
							case '2':   ild.viewing_angle.y += one_degree;
										break;
							//----------------------------------------------------
							case '@':   ild.viewing_angle.y -= one_degree;
										break;
							//----------------------------------------------------
							case '3':   ild.viewing_angle.z += one_degree;
										break;
							//----------------------------------------------------
							case '#':   ild.viewing_angle.z -= one_degree;
										break;
							//----------------------------------------------------
							case '4':   ild.viewing_angle += one_degree;
										break;
							//----------------------------------------------------
							case '$':   ild.viewing_angle -= one_degree;
										break;
							//----------------------------------------------------
							case '5':   ild.viewing_angle   = 0;
										cout << "front" << endl;
										break;
							//----------------------------------------------------
							case '%':   ild.viewing_angle.x = 0;
										ild.viewing_angle.y = pi;
										ild.viewing_angle.z = 0;
										cout << "back" << endl;
										break;
							//----------------------------------------------------
							case '6':   ild.viewing_angle.x = 0;
										ild.viewing_angle.y = half_pi;
										ild.viewing_angle.z = 0;
										cout << "left side" << endl;
										break;
							//----------------------------------------------------
							case '^':   ild.viewing_angle.x = 0;
										ild.viewing_angle.y = -half_pi;
										ild.viewing_angle.z = 0;
										cout << "right side" << endl;
										break;
							//----------------------------------------------------
							case '7':   ild.viewing_angle.x = half_pi;
										ild.viewing_angle.y = 0;
										ild.viewing_angle.z = 0;
										cout << "top" << endl;
										break;
							//----------------------------------------------------
							case '&':   ild.viewing_angle.x = -half_pi;
										ild.viewing_angle.y = 0;
										ild.viewing_angle.z = 0;
										cout << "bottom" << endl;
										break;
							//----------------------------------------------------
						}
						ild.display();
					}
					cout << "draw, move, recolor points and segments mode terminated" << endl;
					show_hot_keys();
					ild.show_cursors = false;
					ild.display();
					break;
				}
				//----------------------------------------------------------------




				












				//----------------------------------------------------------------
				case EZFB_CTRL_K: // enter selection
				{
					short   offset;
                
					ild.show_cursors         = true;
					ild.show_arbitrary_point = true;
					ild.display();
						 // "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
					cout << EZ_ANSI_BOLD
							"\n\n\n\n\n\n"
			EZ_ANSI_RED     "move, scale, rotate segments mode activated\n"
			EZ_ANSI_YELLOW  "      \\||       : select-start to first | select-end to last\n"
							"[|{    |    ]|} : select-start | end        forward | backward one point\n"
							"i|I | o|O | u|U : select-start | end | both forward | backward per lit segment\n"
			EZ_ANSI_GREEN   "       c        : copy   selected-points (in place) to end of point set\n"
			EZ_ANSI_CYAN   	" q  |  w  |  e  : set    selected-points X|Y|Z coordinate center\n"
							"x|X | y|Y | z|Z : move   selected-points X|Y|Z +|-" << move_points_per_key << " points\n"
							"f|F    |    g|G : rotate selected-points +|-1 around segment center Z|Y\n"
							"8|* | 9|( | 0|) : scale  selected-points X|Y|Z +|-1% from segment center\n"
							"      -|_       : scale  selected-points  XYZ  +|-1% from segment center\n"
			EZ_ANSI_BLUE    " j  |  k  |  l  : set    arbitrary point X|Y|Z coordinate\n"
							"a|A | s|S | d|D : move   arbitrary point X|Y|Z +|-" << move_points_per_key << " points\n"
							"r|R    |    t|T : rotate selected-points +|-1 around arbitrary point Z|Y\n"
							"v|V | b|B | n|N : scale  selected-points X|Y|Z +|-1% from arbitrary point\n"
							"      m|M       : scale  selected-points  XYZ  +|-1% from arbitrary point\n"
			EZ_ANSI_MAGENTA "      /|?       : show stats of frame set | current frame\n"
			EZ_ANSI_RED     "     [ESC]      : exit move, scale, rotate segments mode\n"
			EZ_ANSI_WHITE   "ARROW_KEYs and view rotation are active"
			EZ_ANSI_RESET   << endl;

					while((key = ezfb_get_key()) != EZFB_ESCAPE_KEY) // from ezfb_tty.c
					{   // ezfb_get_key() accepts a single key stroke with no return
						switch(key)
						{
							//----------------------------------------------------
							case '/': // show frame set stats
										ild.tell();
										cout << "\narbitrary point" << endl;
										ild.arbitrary_point.tell();
										cout << endl;
										break;
							//----------------------------------------------------
							case '?': // show frame stats
										ild.current_frame().stats(ild.show_cursors);
										cout << "\narbitrary point" << endl;
										ild.arbitrary_point.tell();
										cout << endl;
										break;
							//----------------------------------------------------
							case '[':   ild.current_frame().increment_point_select_start();
										break;
							case '{':   ild.current_frame().deccrement_point_select_start();
										break;
							case'\\':   ild.current_frame().point_select_start = 0;
										cout << "first point" << endl;
										break;
							//----------------------------------------------------
							case ']':   ild.current_frame().increment_point_select_end();
										break;
							case '}':   ild.current_frame().deccrement_point_select_end();
										break;
							case '|':   ild.current_frame().point_select_end = (ild.current_frame().number_of_points() - 1);
										cout << "last point" << endl;
										break;
							//----------------------------------------------------
							case 'u':   ild.current_frame().next_segment_select();
										//cout << "next lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'U':   ild.current_frame().previous_segment_select();
										//cout << "previous lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'i':   ild.current_frame().next_segment_select_start();
										//cout << "next lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'I':   ild.current_frame().previous_segment_select_start();
										//cout << "previous lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'o':   ild.current_frame().next_segment_select_end();
										//cout << "next lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'O':   ild.current_frame().previous_segment_select_end();
										//cout << "previous lit segment" << endl;
										break;
							//----------------------------------------------------
							case 'c':   ild.current_frame() += ild.current_frame().selected_segment();
										cout << "copied "
											 << ild.current_frame().selected_segment().size()
											 << " points to end of point set"
											 << endl;
										break;
							//----------------------------------------------------            
							case 'j':  	cout << "current position of arbitrary point" << endl;
										ild.arbitrary_point.tell();
										cout << "enter new value for X : ";
										ild.arbitrary_point.x = ezfb_get_int();
										ild.arbitrary_point.tell();
										break;
							//----------------------------------------------------            
							case 'k':  	cout << "current position of arbitrary point" << endl;
										ild.arbitrary_point.tell();
										cout << "enter new value for Y : ";
										ild.arbitrary_point.y = ezfb_get_int();
										ild.arbitrary_point.tell();
										break;
							//----------------------------------------------------            
							case 'l':  	cout << "current position of arbitrary point" << endl;
										ild.arbitrary_point.tell();
										cout << "enter new value for Z : ";
										ild.arbitrary_point.z = ezfb_get_int();
										ild.arbitrary_point.tell();
										break;
							//----------------------------------------------------            
							case 'q':  	cout << "current center of selected-points" << endl;
										ild.current_frame().selected_segment().find_center().tell();
										cout << "enter new value for X : ";
										offset = ezfb_get_int();
										ild.current_frame().move_selection(ez_3D_float(   offset - ild.current_frame().selected_segment().find_center().x
																						, 0
																						, 0
																					  )
																		  );
										ild.current_frame().selected_segment().find_center().tell();
										break;
							//----------------------------------------------------            
							case 'w':  	cout << "current center of selected-points" << endl;
										ild.current_frame().selected_segment().find_center().tell();
										cout << "enter new value for Y : ";
										offset = ezfb_get_int();
										ild.current_frame().move_selection(ez_3D_float(   0
																						, offset - ild.current_frame().selected_segment().find_center().y
																						, 0
																					  )
																		  );
										ild.current_frame().selected_segment().find_center().tell();
										break;
							//----------------------------------------------------            
							case 'e':  	cout << "current center of selected-points" << endl;
										ild.current_frame().selected_segment().find_center().tell();
										cout << "enter new value for Z : ";
										offset = ezfb_get_int();
										ild.current_frame().move_selection(ez_3D_float(   0
																						, 0
																						, offset - ild.current_frame().selected_segment().find_center().z
																					  )
																		  );
										ild.current_frame().selected_segment().find_center().tell();
										break;
							//----------------------------------------------------
							case 'x':   ild.current_frame().move_selection(ez_3D_float(move_points_per_key, 0, 0));
										break;
							//----------------------------------------------------
							case 'X':   ild.current_frame().move_selection(ez_3D_float(-move_points_per_key, 0, 0));
										break;
							//----------------------------------------------------
							case 'y':   ild.current_frame().move_selection(ez_3D_float(0, move_points_per_key, 0));
										break;
							//----------------------------------------------------
							case 'Y':   ild.current_frame().move_selection(ez_3D_float(0, -move_points_per_key, 0));
										break;
							//----------------------------------------------------
							case 'z':   ild.current_frame().move_selection(ez_3D_float(0, 0, move_points_per_key));
										break;
							//----------------------------------------------------
							case 'Z':   ild.current_frame().move_selection(ez_3D_float(0, 0,-move_points_per_key));
										break;
							//----------------------------------------------------
							case 'f':   ild.current_frame().rotate_selection(ez_3D_float(0, 0, one_degree));
										break;
							//----------------------------------------------------
							case 'F':   ild.current_frame().rotate_selection(ez_3D_float(0, 0, -one_degree));
										break;
							//----------------------------------------------------
							case 'g':   ild.current_frame().rotate_selection(ez_3D_float(0, one_degree, 0));
										break;
							//----------------------------------------------------
							case 'G':   ild.current_frame().rotate_selection(ez_3D_float(0,-one_degree, 0));
										break;
							//----------------------------------------------------
							case 'r':   ild.current_frame().rotate_selection_around_arbitrary_point(ez_3D_float(0, 0, one_degree));
										break;
							//----------------------------------------------------
							case 'R':   ild.current_frame().rotate_selection_around_arbitrary_point(ez_3D_float(0, 0, -one_degree));
										break;
							//----------------------------------------------------
							case 't':   ild.current_frame().rotate_selection_around_arbitrary_point(ez_3D_float(0, one_degree, 0));
										break;
							//----------------------------------------------------
							case 'T':   ild.current_frame().rotate_selection_around_arbitrary_point(ez_3D_float(0, -one_degree, 0));
										break;
							//----------------------------------------------------
							case '8':   ild.current_frame().scale_selection(ez_3D_float(1.01, 1, 1));
										break;
							//----------------------------------------------------
							case '*':   ild.current_frame().scale_selection(ez_3D_float(0.99, 1, 1));
										break;
							//----------------------------------------------------
							case '9':   ild.current_frame().scale_selection(ez_3D_float(1, 1.01, 1));
										break;
							//----------------------------------------------------
							case '(':   ild.current_frame().scale_selection(ez_3D_float(1, 0.99, 1));
										break;
							//----------------------------------------------------
							case '0':   ild.current_frame().scale_selection(ez_3D_float(1, 1, 1.01));
										break;
							//----------------------------------------------------
							case ')':   ild.current_frame().scale_selection(ez_3D_float(1, 1, 0.99));
										break;
							//----------------------------------------------------
							case '-':   ild.current_frame().scale_selection(ez_3D_float(1.01, 1.01, 1.01));
										break;
							//----------------------------------------------------
							case '_':   ild.current_frame().scale_selection(ez_3D_float(0.99, 0.99, 0.99));
										break;
							//----------------------------------------------------
							case 'v':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(1.01, 1, 1));
										break;
							//----------------------------------------------------
							case 'V':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(0.99, 1, 1));
										break;
							//----------------------------------------------------
							case 'b':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(1, 1.01, 1));
										break;
							//----------------------------------------------------
							case 'B':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(1, 0.99, 1));
										break;
							//----------------------------------------------------
							case 'n':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(1, 1, 1.01));
										break;
							//----------------------------------------------------
							case 'N':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(1, 1, 0.99));
										break;
							//----------------------------------------------------
							case 'm':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(1.01, 1.01, 1.01));
										break;
							//----------------------------------------------------
							case 'M':   ild.current_frame().scale_selection_around_arbitrary_point(ez_3D_float(0.99, 0.99, 0.99));
										break;
							//----------------------------------------------------
							case 'a':   ild.arbitrary_point.x += move_points_per_key;
										break;
							//----------------------------------------------------
							case 'A':   ild.arbitrary_point.x -= move_points_per_key;
										break;
							//----------------------------------------------------
							case 's':   ild.arbitrary_point.y += move_points_per_key;
										break;
							//----------------------------------------------------
							case 'S':   ild.arbitrary_point.y -= move_points_per_key;
										break;
							//----------------------------------------------------
							case 'd':   ild.arbitrary_point.z += move_points_per_key;
										break;
							//----------------------------------------------------
							case 'D':   ild.arbitrary_point.z -= move_points_per_key;
										break;
							//----------------------------------------------------
							case EZFB_ARROW_UP: // display next palette
										ild.target_palette_index++;
										if(ild.target_palette_index >= ild.number_of_palettes())
											ild.target_palette_index = 0;
										break;
							//----------------------------------------------------
							case EZFB_ARROW_DOWN: // display previous palette
										ild.target_palette_index--;
										if(ild.target_palette_index <= 0)
											ild.target_palette_index = ild.number_of_palettes() - 1;
										break;
							//----------------------------------------------------
							case EZFB_ARROW_RIGHT: // display previous frame
										if(    loop_animate
											|| (ild.frame_index < ild.number_of_frames() - 1)
										  )
											++ild;
										break;
							//----------------------------------------------------
							case EZFB_ARROW_LEFT: // display next frame
										if(    loop_animate
											|| (ild.frame_index > 0)
										  )
											--ild;
										break;
							//----------------------------------------------------
							case '1':   ild.viewing_angle.x += one_degree;
										break;
							//----------------------------------------------------
							case '!':   ild.viewing_angle.x -= one_degree;
										break;
							//----------------------------------------------------
							case '2':   ild.viewing_angle.y += one_degree;
										break;
							//----------------------------------------------------
							case '@':   ild.viewing_angle.y -= one_degree;
										break;
							//----------------------------------------------------
							case '3':   ild.viewing_angle.z += one_degree;
										break;
							//----------------------------------------------------
							case '#':   ild.viewing_angle.z -= one_degree;
										break;
							//----------------------------------------------------
							case '4':   ild.viewing_angle += one_degree;
										break;
							//----------------------------------------------------
							case '$':   ild.viewing_angle -= one_degree;
										break;
							//----------------------------------------------------
							case '5':   ild.viewing_angle   = 0;
										cout << "front" << endl;
										break;
							//----------------------------------------------------
							case '%':   ild.viewing_angle.x = 0;
										ild.viewing_angle.y = pi;
										ild.viewing_angle.z = 0;
										cout << "back" << endl;
										break;
							//----------------------------------------------------
							case '6':   ild.viewing_angle.x = 0;
										ild.viewing_angle.y = half_pi;
										ild.viewing_angle.z = 0;
										cout << "left side" << endl;
										break;
							//----------------------------------------------------
							case '^':   ild.viewing_angle.x = 0;
										ild.viewing_angle.y = -half_pi;
										ild.viewing_angle.z = 0;
										cout << "right side" << endl;
										break;
							//----------------------------------------------------
							case '7':   ild.viewing_angle.x = half_pi;
										ild.viewing_angle.y = 0;
										ild.viewing_angle.z = 0;
										cout << "top" << endl;
										break;
							//----------------------------------------------------
							case '&':   ild.viewing_angle.x = -half_pi;
										ild.viewing_angle.y = 0;
										ild.viewing_angle.z = 0;
										cout << "bottom" << endl;
										break;
							//----------------------------------------------------
						}
						ild.display();
					}
					cout << "move, scale, rotate segments mode terminated" << endl << endl;
					show_hot_keys();
					ild.show_arbitrary_point = false;
					ild.show_cursors         = false;
					ild.display();
					break;
				}
				//----------------------------------------------------------------































            
                        
				//----------------------------------------------------------------
				case EZFB_CTRL_A: // alter frames
				{
					cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
			EZ_ANSI_RED     "    1: flatten z (force to 2D)\n"
			EZ_ANSI_YELLOW  "    2: z order points\n"
			EZ_ANSI_GREEN   "    3: reverse point order\n"
			EZ_ANSI_CYAN   	"    4: mirror\n"
			EZ_ANSI_BLUE    "    5: flip\n"
			EZ_ANSI_MAGENTA "    6: quarter turn clockwise\n"
			EZ_ANSI_RED     "    7: quarter turn counter clockwise\n"
			EZ_ANSI_YELLOW  "    8: remove blank span\n"
			EZ_ANSI_GREEN   "    9: remove angle dwell\n"
			EZ_ANSI_CYAN   	"    0: reduce vectors\n"
			EZ_ANSI_BLUE    "    a: bond segments (reorders segments)\n"
			EZ_ANSI_MAGENTA "    b: minimize points (8,9,0,a)\n"
			EZ_ANSI_RED     "    c: fracture segments\n"
			EZ_ANSI_YELLOW  "    d: reorder segments\n"
			EZ_ANSI_GREEN   "    e: add angle angle dwell\n"
			EZ_ANSI_CYAN   	"    f: add distance span\n"
			EZ_ANSI_BLUE    "    g: optimize\n"
			EZ_ANSI_MAGENTA "[ESC]: cancel action\n"
			EZ_ANSI_WHITE   "[SHIFT] select to apply to all frames:\n"
							"select frame alteration option :"
			EZ_ANSI_RESET	<< endl <<  endl;
					do
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '!'
							  && key != '2'
							  && key != '@'
							  && key != '3'
							  && key != '#'
							  && key != '4'
							  && key != '$'
							  && key != '5'
							  && key != '%'
							  && key != '6'
							  && key != '^'
							  && key != '7'
							  && key != '&'
							  && key != '8'
							  && key != '*'
							  && key != '9'
							  && key != '('
							  && key != '0'
							  && key != ')'
							  && key != 'a'
							  && key != 'A'
							  && key != 'b'
							  && key != 'B'
							  && key != 'c'
							  && key != 'C'
							  && key != 'd'
							  && key != 'D'
							  && key != 'e'
							  && key != 'E'
							  && key != 'f'
							  && key != 'F'
							  && key != 'g'
							  && key != 'G'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						show_hot_keys();
						break;
					}

					cout << (char)key << endl;

					switch(key)
					{
						case '1':	cout << "flatten z" << endl;
									ild.current_frame().flatten_z();
									break;
						//--------------------------------------------------------
						case '!':	ild.flatten_z();
									break;
						//--------------------------------------------------------
						case '2':	cout << "z order points" << endl;
									ild.current_frame().z_order_points(64000);
									break;
						//--------------------------------------------------------
						case '@':	ild.z_order_points(64000);
									break;
						//--------------------------------------------------------
						case '3':	cout << "reverse point order" << endl;
									ild.current_frame().reverse();
									break;
						//--------------------------------------------------------
						case '#':	ild.frame_reverse();
									break;
						//--------------------------------------------------------
						case '4':	cout << "mirror" << endl;
									ild.current_frame().flip(0);
									break;
						//--------------------------------------------------------
						case '$':	ild.flip(0);
									break;
						//--------------------------------------------------------
						case '5':	cout << "flip" << endl;
									ild.current_frame().flip(1);
									break;
						//--------------------------------------------------------
						case '%':	ild.flip(1);
									break;
						//--------------------------------------------------------
						case '6':	cout << "quarter turn clockwise" << endl;
									ild.current_frame().quarter_turn(0, 1);
									break;
						//--------------------------------------------------------
						case '^':	ild.quarter_turn(0, 1);
									break;
						//--------------------------------------------------------
						case '7':	cout << "quarter turn counterclockwise" << endl;
									ild.current_frame().quarter_turn(0, 3);
									break;
						//--------------------------------------------------------
						case '&':	ild.quarter_turn(0, 3);
									break;
						//--------------------------------------------------------
						case '8':   cout << "remove blank span" << endl;
									ild.current_frame().minimize_blank_points();
									break;
						//--------------------------------------------------------
						case '*':   ild.minimize_blank_points();
									break;
						//--------------------------------------------------------
						case '9':   cout << "remove angle dwell" << endl;
									ild.current_frame().remove_dwell_points();
									break;
						//--------------------------------------------------------
						case '(':   ild.remove_dwell_points();
									break;
						//--------------------------------------------------------
						case '0':   cout << "reduce vectors" << endl;
									ild.current_frame().minimize_vectors();
									break;
						//--------------------------------------------------------
						case ')':   ild.minimize_vectors();
									break;
						//--------------------------------------------------------                                
						case 'a':   cout << "bond segments" << endl;
									ild.current_frame().bond_segments();
									break;
						//--------------------------------------------------------
						case 'A':   ild.bond_segments();
									break;
						//--------------------------------------------------------
						case 'b':   cout << "minimize points" << endl;
									ild.current_frame().minimize_points();
									break;
						//--------------------------------------------------------
						case 'B':   ild.minimize_points();
									break;
						//--------------------------------------------------------                                
						case 'c':   cout << "fracture segments" << endl;
									ild.current_frame().fracture_segments();
									break;
						//--------------------------------------------------------
						case 'C':   ild.fracture_segments();
									break;
						//--------------------------------------------------------
						case 'd':   cout << "reorder segments" << endl;
									ild.current_frame().reorder_segments();
									break;
						//--------------------------------------------------------
						case 'D':   ild.reorder_segments();
									break;
						//--------------------------------------------------------
						case 'e':   cout << "add angle dwell" << endl;
									ild.current_frame().add_dwell_points();
									break;
						//--------------------------------------------------------
						case 'E':   ild.add_dwell_points();
									break;
						//--------------------------------------------------------
						case 'f':   cout << "add distance span" << endl;
									ild.current_frame().add_span_points();
									break;
						//--------------------------------------------------------
						case 'F':   ild.add_span_points();
									break;
						//--------------------------------------------------------
						case 'g':   cout << "optimize" << endl;
									ild.current_frame().optimize_points();
									break;
						//--------------------------------------------------------
						case 'G':   ild.optimize_points();
									break;
						//--------------------------------------------------------
					}
					cout << "Alter point order or count, DONE!" << endl;
					break;
				}
				//----------------------------------------------------------------









            
            
                        
























				//----------------------------------------------------------------
				case EZFB_CTRL_U: // 
				{
					cout << "\n\n\n\n\n\n" EZ_ANSI_BOLD
			EZ_ANSI_RED     "    1: delete selected frames\n"
			EZ_ANSI_YELLOW  "    2: trim to selected frames\n"
			EZ_ANSI_GREEN   "    3: copy selected frames to begining of current frame set\n"
			EZ_ANSI_CYAN   	"    4: move selected frames to begining of current frame set\n"
			EZ_ANSI_BLUE    "    5: copy selected frames to end of current frame set\n"
			EZ_ANSI_MAGENTA "    6: move selected frames to end of current frame set\n"
			EZ_ANSI_RED     "    7: delete redundant frames from wave import\n"
			EZ_ANSI_YELLOW  "    8: add empty frame to begining of current frame set\n"
			EZ_ANSI_GREEN   "    9: add empty frame after current frame in current set\n"
			EZ_ANSI_CYAN   	"    0: add empty frame to end of current frame set\n"
			EZ_ANSI_BLUE    "    a: reverse frame set order\n"
			EZ_ANSI_MAGENTA "    >: explode current frame into frame set of segments\n"
			EZ_ANSI_RED     "    <: collapse selected frames into one frame\n"
			EZ_ANSI_YELLOW  "[ESC]: cancel action\n"
			EZ_ANSI_WHITE   "select dimensional modification: \n"
							"[SHIFT] select to apply to all frames:"
			EZ_ANSI_RESET	<< endl;
					do
					{
						key = ezfb_get_key();
					} while (    key != EZFB_ESCAPE_KEY
							  && key != '1'
							  && key != '2'
							  && key != '3'
							  && key != '4'
							  && key != '5'
							  && key != '6'
							  && key != '7'
							  && key != '8'
							  && key != '9'
							  && key != '0'
							  && key != 'a'
							  && key != '>'
							  && key != '<'
							);

					if(key == EZFB_ESCAPE_KEY)
					{
						cout << "action cancelled" << endl;
						show_hot_keys();
						break;
					}

					cout << (char)key << endl;

					switch(key)
					{
						//--------------------------------------------------------
						case '1':   cout << "delete selected frames" << endl;
									ild.delete_selected_frames();
									break;
						//--------------------------------------------------------
						case '2':   cout << "trim to selected frames" << endl;
									ild.trim_to_selected_frames();
									break;
						//--------------------------------------------------------
						case '3':   cout << "copy selected frames to begining of current frame set" << endl;
									ild = (ild.selected_frames() + ild);
									break;
						//--------------------------------------------------------
						case '4':   cout << "move selected frames to begining of current frame set" << endl;
									{
										ez_ilda_frame_set temp(ild.selected_frames());
										ild.delete_selected_frames();
										temp += ild;
										ild = temp;
									}
									break;
						//--------------------------------------------------------
						case '5':   cout << "copy selected frames to end of current frame set" << endl;
									ild += ild.selected_frames();
									break;
						//--------------------------------------------------------
						case '6':   cout << "move selected frames to end of current frame set" << endl;
									{
										ez_ilda_frame_set temp(ild.selected_frames());
										ild.delete_selected_frames();
										ild += temp;
									}
									break;
						//--------------------------------------------------------
						case '7':   cout << "delete redundant frames fram wave import" << endl;
									ild.delete_redundant_frames();
									break;
						//--------------------------------------------------------
						case '8':   cout << "add empty frame to begining of current frame set" << endl;
									{
										ez_ilda_frame_set temp;
										temp += ez_ilda_frame(ild.target_palette_index, 0); // not quite an empty frame!
										ild = temp + ild;
									}
									break;
						//--------------------------------------------------------
						case '9':   cout << "add empty frame after current frame in current frame set" << endl;
									{
										int               i;
										ez_ilda_frame_set temp;
										for(i = 0; i <= ild.frame_index; i++)
											temp += ild[i];
										temp += ez_ilda_frame(ild.target_palette_index, 0);
										for(i = ild.frame_index + 1; i < ild.number_of_frames(); i++)
											temp += ild[i];
										ild = temp;
									}
									break;
						//--------------------------------------------------------
						case '0':   cout << "add empty frame to end of current frame set" << endl;
									ild += ez_ilda_frame(ild.target_palette_index, 0);
									break;
						//--------------------------------------------------------
						case 'a':   cout << "reverse frame set order" << endl;
									ild.reverse();
									break;
						//--------------------------------------------------------
						case '>': // explode current frame
									ild.explode_current_frame();
									break;
						//--------------------------------------------------------
						case '<': // collapse selected frames
									ild.collapse_selected_frames();
									break;
					}
					cout << "Univsal frame set effect, DONE!" << endl;
					break;
				}
				//----------------------------------------------------------------
			} // end switch(key)
			//--------------------------------------------------------------------
			ild.display();
			wipe_input_string(input_string);
		} // while((key = ezfb_get_key()) != EZFB_ESCAPE_KEY)
		//------------------------------------------------------------------------
		cout << "Do you really want to terminate laserboy? y|Y" << endl;
		key = ezfb_get_key();
		switch(key)
		{   //--------------------------------------------------------------------
			default:    cout << "hit the 'h' key for help!" << endl;
						break;
			//--------------------------------------------------------------------
			case 'y':   
			case 'Y':   application_is_running = false;
						break;
			//--------------------------------------------------------------------
		}
		//------------------------------------------------------------------------
	} // while(application_is_running)
    //----------------------------------------------------------------------------






	//----------------------------------------------------------------------------
#if !LASERBOY_DEMO
	cout << "Exit without saving OUT.ild? y|Y" << endl;
	key = ezfb_get_key();
	switch(key)
	{   
		default :
					if(ild.auto_optimize)
						ild.minimize_points();
					ild.save_as_ilda("OUT.ild");
					break;
		//------------------------------------------------------------------------
		case 'y':   
		case 'Y':   break;
	}
#endif
    //----------------------------------------------------------------------------
    cout << "\nlaserboy application terminated!\n\n" << endl;
    //----------------------------------------------------------------------------
    return 0;
}

//################################################################################
//////////////////////////////////////////////////////////////////////////////////
//################################################################################

