//#################################################################################
//
// project.cpp
//
// ~  This file is part of the LaserBoy package   ~
// ~   International Laser Display Association    ~
// ~ ILDA file display and transformations in C++ ~
// ~               built for EZFB                 ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began : October 2003
//
//#################################################################################
#include "ez_ilda.hpp"
#include "ez_wave.hpp"
#include "ez_oss_dsp.hpp"

//#################################################################################
int                  i                                ,
                     argc                             ,
                     span                             ,
                     last                             ,
                     error                            ,
                     short_count                      ,
                     sample_count                     ,
                     wave_out_data_fd                 ,
                     offset_in_max      = -0x80000000 ,
                     offset_in_min      =  0x7FFFFFFF ,
                     offset_applied_max = -0x80000000 ,
                     offset_applied_min =  0x7FFFFFFF ,
                    *offset_in          =  NULL       ,
                    *offset_applied     =  NULL       ;
//---------------------------------------------------------------------------------
char               **argv                             ;
//---------------------------------------------------------------------------------
bool                 inverted           = false       ;
//---------------------------------------------------------------------------------
string               command_string                   ,
                     file_name                        ;
//---------------------------------------------------------------------------------
ifstream             wave_ifstream                    ;
//---------------------------------------------------------------------------------
ofstream             wave_header_ofstream             ;
//---------------------------------------------------------------------------------
ez_wave_header       wave_in_header                   ,
                     wave_out_header                  ;
//---------------------------------------------------------------------------------
oss_dsp              dsp                              ;
//---------------------------------------------------------------------------------
typedef ez_wave_sample *wave_sample_pointer           ;
wave_sample_pointer     roll_over                     ;
wave_sample_pointer    *sample_window                 ;

//#################################################################################
void clean_up()
{
    command_string = "rm -f wave.header wave.data";
    call_command(command_string, ez_ilda_terminate, false);
    return;
}

//#################################################################################
void open_wave_out()
{
    if((wave_out_data_fd = open("wave.data", O_WRONLY | O_CREAT)) == -1)
    {
        perror("wave.data");
        exit(-1);
    }
    //-----------------------------------------------------------------------------
    wave_header_ofstream.open("wave.header", ios::out | ios::binary);
    if(!wave_header_ofstream.is_open())
        exit(-1);
    //-----------------------------------------------------------------------------
    return;
}

//#################################################################################
void open_wave_in()
{
    if(argc > 1) // file name
    {
        file_name = argv[1];
        if(    file_name.size() < 4
            || file_name.substr(file_name.size() - 4) != ".wav"
          )
            file_name += ".wav";
    }
    else
        file_name = "laserboy.wav";
    //-----------------------------------------------------------------------------
    wave_ifstream.open(file_name.c_str(), ios::in | ios::binary);
    if(!wave_ifstream.is_open())
    {
        cerr << file_name << " failed to open for input" << endl << endl;
        exit(-1);
    }
    //-----------------------------------------------------------------------------
    if((error = wave_in_header.from_ifstream(wave_ifstream)))
    {
        cout << endl << "bad wave file : " << error << endl << endl;
        exit(-1);
    }
    wave_in_header.tell();
    wave_out_header = wave_in_header;
    //-----------------------------------------------------------------------------
    offset_in      = new int[wave_in_header.num_channels];
    offset_applied = new int[wave_in_header.num_channels];
    for(i = 0; i < wave_in_header.num_channels; i++)
    {
        offset_in[i]      = wave_in_header.offset[i];
        offset_applied[i] = 0;
    }
    //-----------------------------------------------------------------------------
    return;
}

//#################################################################################
void get_offset_args()
{
    for(i = 0; i < (argc - 2); i++)
        offset_in[i] = atoi(argv[i + 2]);
    return;
}

//#################################################################################
void calculate_applied_offsets()
{
    for(i = 0; i < wave_in_header.num_channels; i++)
    {
        offset_applied[i] = wave_in_header.offset[i] - offset_in[i];

        if(offset_in[i] > offset_in_max)
            offset_in_max = offset_in[i];

        if(offset_in[i] < offset_in_min)
            offset_in_min = offset_in[i];

        if(offset_applied[i] > offset_applied_max)
            offset_applied_max = offset_applied[i];

        if(offset_applied[i] < offset_applied_min)
            offset_applied_min = offset_applied[i];
    }
    //-----------------------------------------------------------------------------
    span = (offset_applied_max - offset_applied_min) + 1;
    last = span - 1;
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "span  : " << span  << endl
         << "last  : " << last  << endl
         << endl;
#endif
    //-------------------------------------------------------------------------
    for(i = 0; i < wave_in_header.num_channels; i++)
    {
        wave_out_header.offset[i] = -(offset_in_max - offset_in[i]);
        offset_applied[i] = offset_applied_max - offset_applied[i];
#if EZ_ILDA_DEBUG_MESSAGES
        cout << "offset_applied[" << i << "] : " << offset_applied[i] << endl;
#endif
    }
    //-------------------------------------------------------------------------
    sample_window = new wave_sample_pointer[span];
    for(i = 0; i < span; i++)
        sample_window[i] = new ez_wave_sample(wave_in_header.num_channels);
    //-------------------------------------------------------------------------
    return;
}

//#################################################################################
void dsp_setup()
{
    dsp.set_num_channels(wave_in_header.num_channels);
    dsp.set_rate        (wave_in_header.sample_rate);
    dsp.set_vol         (100);
    dsp.set_pcm         ( 81);
    dsp.set_altpcm      (100);
    dsp.set_pc_speaker  (  0);
    dsp.set_mic         (  0);
    dsp.set_CD          (  0);
    dsp.set_line_1      (  0);
    dsp.set_line        (  0);
    dsp.set_synth       (  0);
    dsp.show_mixer();
    cout << "odelay           : " << dsp.get_odelay()           << endl;
    cout << "driver_frag_size : " << dsp.get_driver_frag_size() << endl;
    return;
}

//#################################################################################
void dsp_play()
{
    short_count  =  0;
    sample_count =  0;

    while(true)
    {
        if(last) // span > 1
        {
            roll_over = sample_window[0];
            for(i = 0; i < last; i++)
                sample_window[i] = sample_window[i + 1];
            sample_window[last] = roll_over;
        }

        if(sample_window[last]->from_ifstream(wave_ifstream))
            break;

        sample_count++;

        for(i = 0; i < wave_in_header.num_channels; i++)
            dsp.play_buffer[short_count++] = sample_window[offset_applied[i]]->channel[i];

        if(short_count == DSP_PLAY_BUFFER_SIZE)
        {
            short_count = 0;
            //write(dsp.dsp_fd, (char*)dsp.play_buffer, DSP_PLAY_BUFFER_SIZE * sizeof(short));
            if(last)
                write(wave_out_data_fd , (char*)dsp.play_buffer, DSP_PLAY_BUFFER_SIZE * sizeof(short));
        }     
    }
    //-----------------------------------------------------------------------------
    if(short_count)
    {
        if(last)
            write(wave_out_data_fd, (char*)dsp.play_buffer, short_count * sizeof(short));
        //-------------------------------------------------------------------------
        while(short_count < DSP_PLAY_BUFFER_SIZE)
            dsp.play_buffer[short_count++] = 0;
        //write(dsp.dsp_fd, (char*)dsp.play_buffer, DSP_PLAY_BUFFER_SIZE * sizeof(short));
    }            
    return;
}

//#################################################################################
void dsp_play_inverted()
{
    short_count  =  0;
    sample_count =  0;
    while(true)
    {
        if(last) // span > 1
        {
            roll_over = sample_window[0];
            for(i = 0; i < last; i++)
                sample_window[i] = sample_window[i + 1];
            sample_window[last] = roll_over;
        }

        if(sample_window[last]->from_ifstream_inverted(wave_ifstream))
            break;

        sample_count++;

        for(i = 0; i < wave_in_header.num_channels; i++)
            dsp.play_buffer[short_count++] = sample_window[offset_applied[i]]->channel[i];

        if(short_count == DSP_PLAY_BUFFER_SIZE)
        {
            short_count = 0;
            //write(dsp.dsp_fd, (char*)dsp.play_buffer, DSP_PLAY_BUFFER_SIZE * sizeof(short));
            if(last)
                write(wave_out_data_fd , (char*)dsp.play_buffer, DSP_PLAY_BUFFER_SIZE * sizeof(short));
        }     
    }
    if(short_count)
    {
        if(last)
            write(wave_out_data_fd, (char*)dsp.play_buffer, short_count * sizeof(short));
        //---------------------------------------------------------------------
        while(short_count < DSP_PLAY_BUFFER_SIZE)
            dsp.play_buffer[short_count++] = 0;
        //write(dsp.dsp_fd, (char*)dsp.play_buffer, DSP_PLAY_BUFFER_SIZE * sizeof(short));
    }
    return;
}

//#################################################################################
void save_wave_out()
{
    if(last)
    {
        wave_out_header.num_samples = sample_count;
        wave_out_header.to_ofstream(wave_header_ofstream);
        wave_header_ofstream.close();
        //-------------------------------------------------------------------------
        command_string  = "rm -f " + file_name;
        call_command(command_string, ez_ilda_terminate, false);
        command_string  = "cat wave.header wave.data > " + file_name;
        call_command(command_string, ez_ilda_terminate, false);
    }
    else
        wave_header_ofstream.close();
    return;
}

//#################################################################################
void free_memory()
{
    for(i = 0; i < span; i++)
        delete sample_window[i];
    delete sample_window;
    delete offset_in;
    delete offset_applied;
    return;
}

//#################################################################################
int main(int _argc, char ** _argv)
{
    argc = _argc;
    argv = _argv;
    //-----------------------------------------------------------------------------
    show_tag                  ();
    clean_up                  ();
    open_wave_out             ();
    open_wave_in              ();
    get_offset_args           ();    
    calculate_applied_offsets ();
    dsp_setup                 ();
    //-----------------------------------------------------------------------------
    if(inverted)
        dsp_play_inverted     ();
    //-----------------------------------------------------------------------------
    else
        dsp_play              ();
    //-----------------------------------------------------------------------------
    close                     (wave_out_data_fd);
    save_wave_out             ();
    clean_up                  ();
    free_memory               ();
    //-----------------------------------------------------------------------------
    return 0;
}

//#################################################################################
///////////////////////////////////////////////////////////////////////////////////
//#################################################################################




